@@ -1,7 +1,7 @@
 //! Module that defines the public compilation API of [`Engine`].
 
 use crate::parser::{ParseResult, ParseState};
-use crate::{Engine, Scope, AST};
+use crate::{Engine, OptimizationLevel, Scope, AST};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 
@@ -99,45 +99,50 @@ impl Engine {
         ) {
             ast.walk(&mut |path| match path.last().unwrap() {
                 // Collect all `import` statements with a string constant path
-                ASTNode::Stmt(Stmt::Import(Expr::StringConstant(s, _), _, _))
-                    if !resolver.contains_path(s) && !imports.contains(s.as_str()) =>
-                {
-                    imports.insert(s.clone().into());
-                    true
-                }
+                ASTNode::Stmt(Stmt::Import(x, ..)) => match x.0 {
+                    Expr::StringConstant(ref s, ..)
+                        if !resolver.contains_path(s)
+                            && (imports.is_empty() || !imports.contains(s.as_str())) =>
+                    {
+                        imports.insert(s.clone().into());
+                        true
+                    }
+                    _ => true,
+                },
                 _ => true,
             });
         }
 
         let mut ast = self.compile_scripts_with_scope(scope, &[script])?;
 
-        if let Some(ref module_resolver) = self.module_resolver {
-            let mut resolver = StaticModuleResolver::new();
-            let mut imports = BTreeSet::new();
+        let mut resolver = StaticModuleResolver::new();
+        let mut imports = BTreeSet::new();
 
-            collect_imports(&ast, &resolver, &mut imports);
+        collect_imports(&ast, &resolver, &mut imports);
 
-            if !imports.is_empty() {
-                while let Some(path) = imports.iter().next() {
-                    let path = path.clone();
+        if !imports.is_empty() {
+            while let Some(path) = imports.iter().next() {
+                let path = path.clone();
 
-                    match module_resolver.resolve_ast(self, None, &path, crate::Position::NONE) {
-                        Some(Ok(module_ast)) => {
-                            collect_imports(&module_ast, &resolver, &mut imports)
-                        }
-                        Some(err) => return err,
-                        None => (),
-                    }
-
-                    let module =
-                        module_resolver.resolve(self, None, &path, crate::Position::NONE)?;
-                    let module = shared_take_or_clone(module);
-
-                    imports.remove(&path);
-                    resolver.insert(path, module);
+                match self
+                    .module_resolver
+                    .resolve_ast(self, None, &path, crate::Position::NONE)
+                {
+                    Some(Ok(module_ast)) => collect_imports(&module_ast, &resolver, &mut imports),
+                    Some(err) => return err,
+                    None => (),
                 }
-                ast.set_resolver(resolver);
+
+                let module =
+                    self.module_resolver
+                        .resolve(self, None, &path, crate::Position::NONE)?;
+
+                let module = shared_take_or_clone(module);
+
+                imports.remove(&path);
+                resolver.insert(path, module);
             }
+            ast.set_resolver(resolver);
         }
 
         Ok(ast)
@@ -199,8 +204,7 @@ impl Engine {
         self.compile_with_scope_and_optimization_level(
             scope,
             scripts,
-            #[cfg(not(feature = "no_optimize"))]
-            self.optimization_level,
+            self.options.optimization_level,
         )
     }
     /// Join a list of strings and compile into an [`AST`] using own scope at a specific optimization level.
@@ -215,7 +219,7 @@ impl Engine {
         &self,
         scope: &Scope,
         scripts: impl AsRef<[S]>,
-        #[cfg(not(feature = "no_optimize"))] optimization_level: crate::OptimizationLevel,
+        optimization_level: OptimizationLevel,
     ) -> ParseResult<AST> {
         let (stream, tokenizer_control) = self.lex_raw(
             scripts.as_ref(),
@@ -226,7 +230,6 @@ impl Engine {
             &mut stream.peekable(),
             &mut state,
             scope,
-            #[cfg(not(feature = "no_optimize"))]
             optimization_level,
         )
     }
@@ -302,8 +305,7 @@ impl Engine {
             &mut peekable,
             &mut state,
             scope,
-            #[cfg(not(feature = "no_optimize"))]
-            self.optimization_level,
+            self.options.optimization_level,
         )
     }
     /// Parse a JSON string into an [object map][crate::Map].
@@ -400,7 +402,9 @@ impl Engine {
                 &mut state,
                 &scope,
                 #[cfg(not(feature = "no_optimize"))]
-                crate::OptimizationLevel::None,
+                OptimizationLevel::None,
+                #[cfg(feature = "no_optimize")]
+                OptimizationLevel::default(),
             )?;
             if has_null {
                 scope.push_constant("null", ());
