@@ -3,7 +3,9 @@
 use crate::eval::{EvalState, GlobalRuntimeState};
 use crate::parser::ParseState;
 use crate::types::dynamic::Variant;
-use crate::{Dynamic, Engine, Module, Position, RhaiResult, RhaiResultOf, Scope, AST, ERR};
+use crate::{
+    Dynamic, Engine, Module, OptimizationLevel, Position, RhaiResult, RhaiResultOf, Scope, AST, ERR,
+};
 use std::any::type_name;
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
@@ -32,8 +34,9 @@ impl Engine {
     /// ## Constants Propagation
     ///
     /// If not [`OptimizationLevel::None`][crate::OptimizationLevel::None], constants defined within
-    /// the scope are propagated throughout the script _including_ functions. This allows functions
-    /// to be optimized based on dynamic global constants.
+    /// the scope are propagated throughout the script _including_ functions.
+    ///
+    /// This allows functions to be optimized based on dynamic global constants.
     ///
     /// # Example
     ///
@@ -64,8 +67,7 @@ impl Engine {
         let ast = self.compile_with_scope_and_optimization_level(
             scope,
             &[script],
-            #[cfg(not(feature = "no_optimize"))]
-            self.optimization_level,
+            self.options.optimization_level,
         )?;
         self.eval_ast_with_scope(scope, &ast)
     }
@@ -122,7 +124,9 @@ impl Engine {
             &mut state,
             scope,
             #[cfg(not(feature = "no_optimize"))]
-            crate::OptimizationLevel::None,
+            OptimizationLevel::None,
+            #[cfg(feature = "no_optimize")]
+            OptimizationLevel::default(),
         )?;
 
         self.eval_ast_with_scope(scope, &ast)
@@ -184,19 +188,15 @@ impl Engine {
         scope: &mut Scope,
         ast: &AST,
     ) -> RhaiResultOf<T> {
-        let global = &mut GlobalRuntimeState::new();
+        let global = &mut GlobalRuntimeState::new(self);
 
         let result = self.eval_ast_with_scope_raw(scope, global, ast, 0)?;
 
         let typ = self.map_type_name(result.type_name());
 
         result.try_cast::<T>().ok_or_else(|| {
-            ERR::ErrorMismatchOutputType(
-                self.map_type_name(type_name::<T>()).into(),
-                typ.into(),
-                Position::NONE,
-            )
-            .into()
+            let t = self.map_type_name(type_name::<T>()).into();
+            ERR::ErrorMismatchOutputType(t, typ.into(), Position::NONE).into()
         })
     }
     /// Evaluate an [`AST`] with own scope.
@@ -227,9 +227,9 @@ impl Engine {
             ast.as_ref(),
         ];
         let lib = if lib.first().map(|m: &&Module| m.is_empty()).unwrap_or(true) {
-            &lib[0..0]
+            &[]
         } else {
-            &lib
+            &lib[..]
         };
         self.eval_global_statements(scope, global, &mut state, statements, lib, level)
     }
