@@ -1,9 +1,8 @@
 //! Module defining script expressions.
 
-use super::{ASTNode, Ident, Stmt, StmtBlock};
+use super::{ASTFlags, ASTNode, Ident, Stmt, StmtBlock};
 use crate::engine::{KEYWORD_FN_PTR, OP_EXCLUSIVE_RANGE, OP_INCLUSIVE_RANGE};
 use crate::func::hashing::ALT_ZERO_HASH;
-use crate::module::Namespace;
 use crate::tokenizer::Token;
 use crate::types::dynamic::Union;
 use crate::{calc_fn_hash, Dynamic, FnPtr, Identifier, ImmutableString, Position, StaticVec, INT};
@@ -21,7 +20,7 @@ use std::{
 use std::str::FromStr;
 
 #[cfg(not(feature = "no_float"))]
-use num_traits::Float;
+use num_traits::float::FloatCore as Float;
 
 /// _(internals)_ A binary expression.
 /// Exported under the `internals` feature only.
@@ -33,6 +32,16 @@ pub struct BinaryExpr {
     pub rhs: Expr,
 }
 
+impl From<(Expr, Expr)> for BinaryExpr {
+    #[inline(always)]
+    fn from(value: (Expr, Expr)) -> Self {
+        Self {
+            lhs: value.0,
+            rhs: value.1,
+        }
+    }
+}
+
 /// _(internals)_ A custom syntax expression.
 /// Exported under the `internals` feature only.
 #[derive(Debug, Clone, Hash)]
@@ -156,37 +165,52 @@ impl FnCallHashes {
 
 /// _(internals)_ A function call.
 /// Exported under the `internals` feature only.
-#[derive(Debug, Clone, Default, Hash)]
+#[derive(Clone, Default, Hash)]
 pub struct FnCallExpr {
     /// Namespace of the function, if any.
-    pub namespace: Option<Namespace>,
+    #[cfg(not(feature = "no_module"))]
+    pub namespace: super::Namespace,
     /// Function name.
     pub name: Identifier,
     /// Pre-calculated hashes.
     pub hashes: FnCallHashes,
     /// List of function call argument expressions.
     pub args: StaticVec<Expr>,
-    /// List of function call arguments that are constants.
-    ///
-    /// Any arguments in `args` that is [`Expr::Stack`] indexes into this
-    /// array to find the constant for use as its argument value.
-    ///
-    /// # Notes
-    ///
-    /// Constant arguments are very common in function calls, and keeping each constant in
-    /// an [`Expr::DynamicConstant`] involves an additional allocation.  Keeping the constant
-    /// values in an inlined array avoids these extra allocations.
-    pub constants: StaticVec<Dynamic>,
     /// Does this function call capture the parent scope?
     pub capture_parent_scope: bool,
+    /// [Position] of the function name.
+    pub pos: Position,
+}
+
+impl fmt::Debug for FnCallExpr {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let mut ff = f.debug_struct("FnCallExpr");
+        #[cfg(not(feature = "no_module"))]
+        if !self.namespace.is_empty() {
+            ff.field("namespace", &self.namespace);
+        }
+        if self.capture_parent_scope {
+            ff.field("capture_parent_scope", &self.capture_parent_scope);
+        }
+        ff.field("hash", &self.hashes)
+            .field("name", &self.name)
+            .field("args", &self.args);
+        ff.field("pos", &self.pos);
+        ff.finish()
+    }
 }
 
 impl FnCallExpr {
     /// Does this function call contain a qualified namespace?
+    ///
+    /// Always `false` under `no_module`.
     #[inline(always)]
     #[must_use]
-    pub const fn is_qualified(&self) -> bool {
-        self.namespace.is_some()
+    pub fn is_qualified(&self) -> bool {
+        #[cfg(not(feature = "no_module"))]
+        return !self.namespace.is_empty();
+        #[cfg(feature = "no_module")]
+        return false;
     }
     /// Convert this into an [`Expr::FnCall`].
     #[inline(always)]
@@ -331,8 +355,6 @@ pub enum Expr {
     /// Integer constant.
     IntegerConstant(INT, Position),
     /// Floating-point constant.
-    ///
-    /// Not available under `no_float`.
     #[cfg(not(feature = "no_float"))]
     FloatConstant(FloatWrapper<crate::FLOAT>, Position),
     /// Character constant.
@@ -350,38 +372,38 @@ pub enum Expr {
     ),
     /// ()
     Unit(Position),
-    /// Variable access - optional short index, position, (optional index, optional (hash, modules), variable name)
+    /// Variable access - (optional long index, namespace, namespace hash, variable name), optional short index, position
     ///
     /// The short index is [`u8`] which is used when the index is <= 255, which should be the vast
     /// majority of cases (unless there are more than 255 variables defined!).
     /// This is to avoid reading a pointer redirection during each variable access.
     Variable(
+        #[cfg(not(feature = "no_module"))]
+        Box<(Option<NonZeroUsize>, super::Namespace, u64, Identifier)>,
+        #[cfg(feature = "no_module")] Box<(Option<NonZeroUsize>, (), u64, Identifier)>,
         Option<NonZeroU8>,
         Position,
-        Box<(Option<NonZeroUsize>, Option<(Namespace, u64)>, Identifier)>,
     ),
     /// Property access - ((getter, hash), (setter, hash), prop)
     Property(
-        Box<(
-            (Identifier, u64),
-            (Identifier, u64),
-            (ImmutableString, Position),
-        )>,
+        Box<((Identifier, u64), (Identifier, u64), ImmutableString)>,
+        Position,
     ),
-    /// Stack slot for function calls.  See [`FnCallExpr`] for more details.
-    ///
-    /// This variant does not map to any language structure.  It is used in function calls with
-    /// constant arguments where the `usize` number indexes into an array containing a list of
-    /// constant arguments for the function call.
-    Stack(usize, Position),
+    /// xxx `.` method `(` expr `,` ... `)`
+    MethodCall(Box<FnCallExpr>, Position),
     /// { [statement][Stmt] ... }
     Stmt(Box<StmtBlock>),
     /// func `(` expr `,` ... `)`
     FnCall(Box<FnCallExpr>, Position),
-    /// lhs `.` rhs - bool variable is a dummy
-    Dot(Box<BinaryExpr>, bool, Position),
-    /// expr `[` expr `]` - boolean indicates whether the dotting/indexing chain stops
-    Index(Box<BinaryExpr>, bool, Position),
+    /// lhs `.` rhs
+    Dot(Box<BinaryExpr>, ASTFlags, Position),
+    /// lhs `[` rhs `]`
+    ///
+    /// ### Flags
+    ///
+    /// [`NONE`][ASTFlags::NONE] = recurse into the indexing chain
+    /// [`BREAK`][ASTFlags::BREAK] = terminate the indexing chain
+    Index(Box<BinaryExpr>, ASTFlags, Position),
     /// lhs `&&` rhs
     And(Box<BinaryExpr>, Position),
     /// lhs `||` rhs
@@ -399,65 +421,64 @@ impl Default for Expr {
 
 impl fmt::Debug for Expr {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let mut display_pos = self.position();
+        let mut display_pos = format!(" @ {:?}", self.start_position());
 
         match self {
-            Self::DynamicConstant(value, _) => write!(f, "{:?}", value),
-            Self::BoolConstant(value, _) => write!(f, "{:?}", value),
-            Self::IntegerConstant(value, _) => write!(f, "{:?}", value),
+            Self::DynamicConstant(value, ..) => write!(f, "{:?}", value),
+            Self::BoolConstant(value, ..) => write!(f, "{:?}", value),
+            Self::IntegerConstant(value, ..) => write!(f, "{:?}", value),
             #[cfg(not(feature = "no_float"))]
-            Self::FloatConstant(value, _) => write!(f, "{:?}", value),
-            Self::CharConstant(value, _) => write!(f, "{:?}", value),
-            Self::StringConstant(value, _) => write!(f, "{:?}", value),
-            Self::Unit(_) => f.write_str("()"),
+            Self::FloatConstant(value, ..) => write!(f, "{:?}", value),
+            Self::CharConstant(value, ..) => write!(f, "{:?}", value),
+            Self::StringConstant(value, ..) => write!(f, "{:?}", value),
+            Self::Unit(..) => f.write_str("()"),
 
-            Self::InterpolatedString(x, _) => {
+            Self::InterpolatedString(x, ..) => {
                 f.write_str("InterpolatedString")?;
                 return f.debug_list().entries(x.iter()).finish();
             }
-            Self::Array(x, _) => {
+            Self::Array(x, ..) => {
                 f.write_str("Array")?;
                 f.debug_list().entries(x.iter()).finish()
             }
-            Self::Map(x, _) => {
+            Self::Map(x, ..) => {
                 f.write_str("Map")?;
                 f.debug_map()
                     .entries(x.0.iter().map(|(k, v)| (k, v)))
                     .finish()
             }
-            Self::Variable(i, _, x) => {
+            Self::Variable(x, i, ..) => {
                 f.write_str("Variable(")?;
-                if let Some((_, ref namespace)) = x.1 {
-                    write!(f, "{}{}", namespace, Token::DoubleColon.literal_syntax())?
+
+                #[cfg(not(feature = "no_module"))]
+                if !x.1.is_empty() {
+                    write!(f, "{}{}", x.1, Token::DoubleColon.literal_syntax())?;
+                    let pos = x.1.position();
+                    if !pos.is_none() {
+                        display_pos = format!(" @ {:?}", pos);
+                    }
                 }
-                f.write_str(&x.2)?;
+                f.write_str(&x.3)?;
                 if let Some(n) = i.map_or_else(|| x.0, |n| NonZeroUsize::new(n.get() as usize)) {
-                    write!(f, " #{}", n)?
+                    write!(f, " #{}", n)?;
                 }
                 f.write_str(")")
             }
-            Self::Property(x) => write!(f, "Property({})", (x.2).0),
-            Self::Stack(x, _) => write!(f, "StackSlot({})", x),
+            Self::Property(x, ..) => write!(f, "Property({})", x.2),
+            Self::MethodCall(x, ..) => f.debug_tuple("MethodCall").field(x).finish(),
             Self::Stmt(x) => {
+                let pos = x.span();
+                if !pos.is_none() {
+                    display_pos = format!(" @ {:?}", pos);
+                }
                 f.write_str("ExprStmtBlock")?;
                 f.debug_list().entries(x.iter()).finish()
             }
-            Self::FnCall(x, _) => {
-                let mut ff = f.debug_struct("FnCall");
-                x.namespace.as_ref().map(|ns| ff.field("namespace", ns));
-                ff.field("name", &x.name)
-                    .field("hash", &x.hashes)
-                    .field("args", &x.args);
-                if !x.constants.is_empty() {
-                    ff.field("constants", &x.constants);
-                }
-                if x.capture_parent_scope {
-                    ff.field("capture_parent_scope", &x.capture_parent_scope);
-                }
-                ff.finish()
-            }
+            Self::FnCall(x, ..) => fmt::Debug::fmt(x, f),
             Self::Index(x, term, pos) => {
-                display_pos = *pos;
+                if !pos.is_none() {
+                    display_pos = format!(" @ {:?}", pos);
+                }
 
                 f.debug_struct("Index")
                     .field("lhs", &x.lhs)
@@ -467,26 +488,28 @@ impl fmt::Debug for Expr {
             }
             Self::Dot(x, _, pos) | Self::And(x, pos) | Self::Or(x, pos) => {
                 let op_name = match self {
-                    Self::Dot(_, _, _) => "Dot",
-                    Self::And(_, _) => "And",
-                    Self::Or(_, _) => "Or",
+                    Self::Dot(..) => "Dot",
+                    Self::And(..) => "And",
+                    Self::Or(..) => "Or",
                     expr => unreachable!(
                         "Self::Dot or Self::And or Self::Or expected but gets {:?}",
                         expr
                     ),
                 };
 
-                display_pos = *pos;
+                if !pos.is_none() {
+                    display_pos = format!(" @ {:?}", pos);
+                }
 
                 f.debug_struct(op_name)
                     .field("lhs", &x.lhs)
                     .field("rhs", &x.rhs)
                     .finish()
             }
-            Self::Custom(x, _) => f.debug_tuple("Custom").field(x).finish(),
+            Self::Custom(x, ..) => f.debug_tuple("Custom").field(x).finish(),
         }?;
 
-        display_pos.debug_print(f)
+        f.write_str(&display_pos)
     }
 }
 
@@ -498,24 +521,24 @@ impl Expr {
     #[must_use]
     pub fn get_literal_value(&self) -> Option<Dynamic> {
         Some(match self {
-            Self::DynamicConstant(x, _) => x.as_ref().clone(),
-            Self::IntegerConstant(x, _) => (*x).into(),
+            Self::DynamicConstant(x, ..) => x.as_ref().clone(),
+            Self::IntegerConstant(x, ..) => (*x).into(),
             #[cfg(not(feature = "no_float"))]
-            Self::FloatConstant(x, _) => (*x).into(),
-            Self::CharConstant(x, _) => (*x).into(),
-            Self::StringConstant(x, _) => x.clone().into(),
-            Self::BoolConstant(x, _) => (*x).into(),
-            Self::Unit(_) => Dynamic::UNIT,
+            Self::FloatConstant(x, ..) => (*x).into(),
+            Self::CharConstant(x, ..) => (*x).into(),
+            Self::StringConstant(x, ..) => x.clone().into(),
+            Self::BoolConstant(x, ..) => (*x).into(),
+            Self::Unit(..) => Dynamic::UNIT,
 
             #[cfg(not(feature = "no_index"))]
-            Self::Array(x, _) if self.is_constant() => {
+            Self::Array(x, ..) if self.is_constant() => {
                 let mut arr = crate::Array::with_capacity(x.len());
                 arr.extend(x.iter().map(|v| v.get_literal_value().unwrap()));
                 Dynamic::from_array(arr)
             }
 
             #[cfg(not(feature = "no_object"))]
-            Self::Map(x, _) if self.is_constant() => {
+            Self::Map(x, ..) if self.is_constant() => {
                 Dynamic::from_map(x.0.iter().fold(x.1.clone(), |mut map, (k, v)| {
                     let value_ref = map.get_mut(k.name.as_str()).unwrap();
                     *value_ref = v.get_literal_value().unwrap();
@@ -524,48 +547,46 @@ impl Expr {
             }
 
             // Fn
-            Self::FnCall(ref x, _)
+            Self::FnCall(ref x, ..)
                 if !x.is_qualified() && x.args.len() == 1 && x.name == KEYWORD_FN_PTR =>
             {
-                if let Expr::StringConstant(ref s, _) = x.args[0] {
-                    if let Ok(fn_ptr) = FnPtr::new(s) {
-                        fn_ptr.into()
-                    } else {
-                        return None;
-                    }
+                if let Expr::StringConstant(ref s, ..) = x.args[0] {
+                    FnPtr::new(s).ok()?.into()
                 } else {
                     return None;
                 }
             }
 
             // Binary operators
-            Self::FnCall(x, _) if !x.is_qualified() && x.args.len() == 2 => match x.name.as_str() {
-                // x..y
-                OP_EXCLUSIVE_RANGE => {
-                    if let Expr::IntegerConstant(ref start, _) = x.args[0] {
-                        if let Expr::IntegerConstant(ref end, _) = x.args[1] {
-                            (*start..*end).into()
+            Self::FnCall(x, ..) if !x.is_qualified() && x.args.len() == 2 => {
+                match x.name.as_str() {
+                    // x..y
+                    OP_EXCLUSIVE_RANGE => {
+                        if let Expr::IntegerConstant(ref start, ..) = x.args[0] {
+                            if let Expr::IntegerConstant(ref end, ..) = x.args[1] {
+                                (*start..*end).into()
+                            } else {
+                                return None;
+                            }
                         } else {
                             return None;
                         }
-                    } else {
-                        return None;
                     }
-                }
-                // x..=y
-                OP_INCLUSIVE_RANGE => {
-                    if let Expr::IntegerConstant(ref start, _) = x.args[0] {
-                        if let Expr::IntegerConstant(ref end, _) = x.args[1] {
-                            (*start..=*end).into()
+                    // x..=y
+                    OP_INCLUSIVE_RANGE => {
+                        if let Expr::IntegerConstant(ref start, ..) = x.args[0] {
+                            if let Expr::IntegerConstant(ref end, ..) = x.args[1] {
+                                (*start..=*end).into()
+                            } else {
+                                return None;
+                            }
                         } else {
                             return None;
                         }
-                    } else {
-                        return None;
                     }
+                    _ => return None,
                 }
-                _ => return None,
-            },
+            }
 
             _ => return None,
         })
@@ -575,32 +596,33 @@ impl Expr {
     #[must_use]
     pub fn from_dynamic(value: Dynamic, pos: Position) -> Self {
         match value.0 {
-            Union::Unit(_, _, _) => Self::Unit(pos),
-            Union::Bool(b, _, _) => Self::BoolConstant(b, pos),
-            Union::Str(s, _, _) => Self::StringConstant(s, pos),
-            Union::Char(c, _, _) => Self::CharConstant(c, pos),
-            Union::Int(i, _, _) => Self::IntegerConstant(i, pos),
+            Union::Unit(..) => Self::Unit(pos),
+            Union::Bool(b, ..) => Self::BoolConstant(b, pos),
+            Union::Str(s, ..) => Self::StringConstant(s, pos),
+            Union::Char(c, ..) => Self::CharConstant(c, pos),
+            Union::Int(i, ..) => Self::IntegerConstant(i, pos),
 
             #[cfg(feature = "decimal")]
-            Union::Decimal(value, _, _) => Self::DynamicConstant(Box::new((*value).into()), pos),
+            Union::Decimal(value, ..) => Self::DynamicConstant(Box::new((*value).into()), pos),
 
             #[cfg(not(feature = "no_float"))]
-            Union::Float(f, _, _) => Self::FloatConstant(f, pos),
+            Union::Float(f, ..) => Self::FloatConstant(f, pos),
 
             #[cfg(not(feature = "no_index"))]
-            Union::Array(a, _, _) => Self::DynamicConstant(Box::new((*a).into()), pos),
+            Union::Array(a, ..) => Self::DynamicConstant(Box::new((*a).into()), pos),
 
             #[cfg(not(feature = "no_object"))]
-            Union::Map(m, _, _) => Self::DynamicConstant(Box::new((*m).into()), pos),
+            Union::Map(m, ..) => Self::DynamicConstant(Box::new((*m).into()), pos),
 
-            Union::FnPtr(f, _, _) if !f.is_curried() => Self::FnCall(
+            Union::FnPtr(f, ..) if !f.is_curried() => Self::FnCall(
                 FnCallExpr {
-                    namespace: None,
+                    #[cfg(not(feature = "no_module"))]
+                    namespace: super::Namespace::NONE,
                     name: KEYWORD_FN_PTR.into(),
                     hashes: calc_fn_hash(f.fn_name(), 1).into(),
-                    args: once(Self::Stack(0, pos)).collect(),
-                    constants: once(f.fn_name().into()).collect(),
+                    args: once(Self::StringConstant(f.fn_name().into(), pos)).collect(),
                     capture_parent_scope: false,
+                    pos,
                 }
                 .into(),
                 pos,
@@ -610,20 +632,32 @@ impl Expr {
         }
     }
     /// Is the expression a simple variable access?
+    ///
+    /// `non_qualified` is ignored under `no_module`.
     #[inline]
     #[must_use]
-    pub(crate) const fn is_variable_access(&self, non_qualified: bool) -> bool {
+    pub(crate) fn is_variable_access(&self, non_qualified: bool) -> bool {
+        let _non_qualified = non_qualified;
+
         match self {
-            Self::Variable(_, _, x) => !non_qualified || x.1.is_none(),
+            #[cfg(not(feature = "no_module"))]
+            Self::Variable(x, ..) if _non_qualified && !x.1.is_empty() => false,
+            Self::Variable(..) => true,
             _ => false,
         }
     }
     /// Return the variable name if the expression a simple variable access.
+    ///
+    /// `non_qualified` is ignored under `no_module`.
     #[inline]
     #[must_use]
     pub(crate) fn get_variable_name(&self, non_qualified: bool) -> Option<&str> {
+        let _non_qualified = non_qualified;
+
         match self {
-            Self::Variable(_, _, x) if !non_qualified || x.1.is_none() => Some(x.2.as_str()),
+            #[cfg(not(feature = "no_module"))]
+            Self::Variable(x, ..) if _non_qualified && !x.1.is_empty() => None,
+            Self::Variable(x, ..) => Some(x.3.as_str()),
             _ => None,
         }
     }
@@ -633,28 +667,49 @@ impl Expr {
     pub const fn position(&self) -> Position {
         match self {
             #[cfg(not(feature = "no_float"))]
-            Self::FloatConstant(_, pos) => *pos,
+            Self::FloatConstant(.., pos) => *pos,
 
-            Self::DynamicConstant(_, pos)
-            | Self::BoolConstant(_, pos)
-            | Self::IntegerConstant(_, pos)
-            | Self::CharConstant(_, pos)
+            Self::DynamicConstant(.., pos)
+            | Self::BoolConstant(.., pos)
+            | Self::IntegerConstant(.., pos)
+            | Self::CharConstant(.., pos)
             | Self::Unit(pos)
-            | Self::StringConstant(_, pos)
-            | Self::Array(_, pos)
-            | Self::Map(_, pos)
-            | Self::Variable(_, pos, _)
-            | Self::Stack(_, pos)
-            | Self::FnCall(_, pos)
-            | Self::Custom(_, pos)
-            | Self::InterpolatedString(_, pos) => *pos,
+            | Self::StringConstant(.., pos)
+            | Self::Array(.., pos)
+            | Self::Map(.., pos)
+            | Self::Variable(.., pos)
+            | Self::And(.., pos)
+            | Self::Or(.., pos)
+            | Self::Index(.., pos)
+            | Self::Dot(.., pos)
+            | Self::Custom(.., pos)
+            | Self::InterpolatedString(.., pos)
+            | Self::Property(.., pos) => *pos,
+
+            Self::FnCall(x, ..) | Self::MethodCall(x, ..) => x.pos,
 
-            Self::Property(x) => (x.2).1,
             Self::Stmt(x) => x.position(),
-
-            Self::And(x, _) | Self::Or(x, _) | Self::Dot(x, _, _) | Self::Index(x, _, _) => {
-                x.lhs.position()
+        }
+    }
+    /// Get the starting [position][Position] of the expression.
+    /// For a binary expression, this will be the left-most LHS instead of the operator.
+    #[inline]
+    #[must_use]
+    pub fn start_position(&self) -> Position {
+        match self {
+            #[cfg(not(feature = "no_module"))]
+            Self::Variable(x, ..) => {
+                if !x.1.is_empty() {
+                    x.1.position()
+                } else {
+                    self.position()
+                }
             }
+            Self::And(x, ..) | Self::Or(x, ..) | Self::Index(x, ..) | Self::Dot(x, ..) => {
+                x.lhs.start_position()
+            }
+            Self::FnCall(.., pos) => *pos,
+            _ => self.position(),
         }
     }
     /// Override the [position][Position] of the expression.
@@ -662,28 +717,28 @@ impl Expr {
     pub fn set_position(&mut self, new_pos: Position) -> &mut Self {
         match self {
             #[cfg(not(feature = "no_float"))]
-            Self::FloatConstant(_, pos) => *pos = new_pos,
+            Self::FloatConstant(.., pos) => *pos = new_pos,
 
-            Self::DynamicConstant(_, pos)
-            | Self::BoolConstant(_, pos)
-            | Self::IntegerConstant(_, pos)
-            | Self::CharConstant(_, pos)
+            Self::DynamicConstant(.., pos)
+            | Self::BoolConstant(.., pos)
+            | Self::IntegerConstant(.., pos)
+            | Self::CharConstant(.., pos)
             | Self::Unit(pos)
-            | Self::StringConstant(_, pos)
-            | Self::Array(_, pos)
-            | Self::Map(_, pos)
-            | Self::And(_, pos)
-            | Self::Or(_, pos)
-            | Self::Dot(_, _, pos)
-            | Self::Index(_, _, pos)
-            | Self::Variable(_, pos, _)
-            | Self::Stack(_, pos)
-            | Self::FnCall(_, pos)
-            | Self::Custom(_, pos)
-            | Self::InterpolatedString(_, pos) => *pos = new_pos,
+            | Self::StringConstant(.., pos)
+            | Self::Array(.., pos)
+            | Self::Map(.., pos)
+            | Self::And(.., pos)
+            | Self::Or(.., pos)
+            | Self::Dot(.., pos)
+            | Self::Index(.., pos)
+            | Self::Variable(.., pos)
+            | Self::FnCall(.., pos)
+            | Self::MethodCall(.., pos)
+            | Self::Custom(.., pos)
+            | Self::InterpolatedString(.., pos)
+            | Self::Property(.., pos) => *pos = new_pos,
 
-            Self::Property(x) => (x.2).1 = new_pos,
-            Self::Stmt(x) => x.set_position(new_pos),
+            Self::Stmt(x) => x.set_position(new_pos, Position::NONE),
         }
 
         self
@@ -695,15 +750,15 @@ impl Expr {
     #[must_use]
     pub fn is_pure(&self) -> bool {
         match self {
-            Self::InterpolatedString(x, _) | Self::Array(x, _) => x.iter().all(Self::is_pure),
+            Self::InterpolatedString(x, ..) | Self::Array(x, ..) => x.iter().all(Self::is_pure),
 
-            Self::Map(x, _) => x.0.iter().map(|(_, v)| v).all(Self::is_pure),
+            Self::Map(x, ..) => x.0.iter().map(|(.., v)| v).all(Self::is_pure),
 
-            Self::And(x, _) | Self::Or(x, _) => x.lhs.is_pure() && x.rhs.is_pure(),
+            Self::And(x, ..) | Self::Or(x, ..) => x.lhs.is_pure() && x.rhs.is_pure(),
 
             Self::Stmt(x) => x.iter().all(Stmt::is_pure),
 
-            Self::Variable(_, _, _) | Self::Stack(_, _) => true,
+            Self::Variable(..) => true,
 
             _ => self.is_constant(),
         }
@@ -712,7 +767,7 @@ impl Expr {
     #[inline(always)]
     #[must_use]
     pub const fn is_unit(&self) -> bool {
-        matches!(self, Self::Unit(_))
+        matches!(self, Self::Unit(..))
     }
     /// Is the expression a constant?
     #[inline]
@@ -720,19 +775,18 @@ impl Expr {
     pub fn is_constant(&self) -> bool {
         match self {
             #[cfg(not(feature = "no_float"))]
-            Self::FloatConstant(_, _) => true,
+            Self::FloatConstant(..) => true,
 
-            Self::DynamicConstant(_, _)
-            | Self::BoolConstant(_, _)
-            | Self::IntegerConstant(_, _)
-            | Self::CharConstant(_, _)
-            | Self::StringConstant(_, _)
-            | Self::Unit(_)
-            | Self::Stack(_, _) => true,
+            Self::DynamicConstant(..)
+            | Self::BoolConstant(..)
+            | Self::IntegerConstant(..)
+            | Self::CharConstant(..)
+            | Self::StringConstant(..)
+            | Self::Unit(..) => true,
 
-            Self::InterpolatedString(x, _) | Self::Array(x, _) => x.iter().all(Self::is_constant),
+            Self::InterpolatedString(x, ..) | Self::Array(x, ..) => x.iter().all(Self::is_constant),
 
-            Self::Map(x, _) => x.0.iter().map(|(_, expr)| expr).all(Self::is_constant),
+            Self::Map(x, ..) => x.0.iter().map(|(.., expr)| expr).all(Self::is_constant),
 
             _ => false,
         }
@@ -749,31 +803,32 @@ impl Expr {
 
         match self {
             #[cfg(not(feature = "no_float"))]
-            Self::FloatConstant(_, _) => false,
+            Self::FloatConstant(..) => false,
 
-            Self::DynamicConstant(_, _)
-            | Self::BoolConstant(_, _)
-            | Self::CharConstant(_, _)
-            | Self::And(_, _)
-            | Self::Or(_, _)
-            | Self::Unit(_) => false,
+            Self::DynamicConstant(..)
+            | Self::BoolConstant(..)
+            | Self::CharConstant(..)
+            | Self::And(..)
+            | Self::Or(..)
+            | Self::Unit(..) => false,
 
-            Self::IntegerConstant(_, _)
-            | Self::StringConstant(_, _)
-            | Self::InterpolatedString(_, _)
-            | Self::FnCall(_, _)
-            | Self::Stmt(_)
-            | Self::Dot(_, _, _)
-            | Self::Index(_, _, _)
-            | Self::Array(_, _)
-            | Self::Map(_, _)
-            | Self::Custom(_, _) => match token {
+            Self::IntegerConstant(..)
+            | Self::StringConstant(..)
+            | Self::InterpolatedString(..)
+            | Self::FnCall(..)
+            | Self::MethodCall(..)
+            | Self::Stmt(..)
+            | Self::Dot(..)
+            | Self::Index(..)
+            | Self::Array(..)
+            | Self::Map(..)
+            | Self::Custom(..) => match token {
                 #[cfg(not(feature = "no_index"))]
                 Token::LeftBracket => true,
                 _ => false,
             },
 
-            Self::Variable(_, _, _) => match token {
+            Self::Variable(..) => match token {
                 #[cfg(not(feature = "no_index"))]
                 Token::LeftBracket => true,
                 Token::LeftParen => true,
@@ -782,14 +837,12 @@ impl Expr {
                 _ => false,
             },
 
-            Self::Property(_) => match token {
+            Self::Property(..) => match token {
                 #[cfg(not(feature = "no_index"))]
                 Token::LeftBracket => true,
                 Token::LeftParen => true,
                 _ => false,
             },
-
-            Self::Stack(_, _) => false,
         }
     }
     /// Recursively walk this expression.
@@ -814,21 +867,21 @@ impl Expr {
                     }
                 }
             }
-            Self::InterpolatedString(x, _) | Self::Array(x, _) => {
+            Self::InterpolatedString(x, ..) | Self::Array(x, ..) => {
                 for e in x.as_ref() {
                     if !e.walk(path, on_node) {
                         return false;
                     }
                 }
             }
-            Self::Map(x, _) => {
-                for (_, e) in &x.0 {
+            Self::Map(x, ..) => {
+                for (.., e) in &x.0 {
                     if !e.walk(path, on_node) {
                         return false;
                     }
                 }
             }
-            Self::Index(x, _, _) | Self::Dot(x, _, _) | Expr::And(x, _) | Expr::Or(x, _) => {
+            Self::Index(x, ..) | Self::Dot(x, ..) | Expr::And(x, ..) | Expr::Or(x, ..) => {
                 if !x.lhs.walk(path, on_node) {
                     return false;
                 }
@@ -836,14 +889,14 @@ impl Expr {
                     return false;
                 }
             }
-            Self::FnCall(x, _) => {
+            Self::FnCall(x, ..) => {
                 for e in &x.args {
                     if !e.walk(path, on_node) {
                         return false;
                     }
                 }
             }
-            Self::Custom(x, _) => {
+            Self::Custom(x, ..) => {
                 for e in &x.inputs {
                     if !e.walk(path, on_node) {
                         return false;
