@@ -1,18 +1,19 @@
 //! Main module defining the script evaluation [`Engine`].
 
 use crate::api::custom_syntax::CustomSyntax;
-use crate::func::native::{OnDebugCallback, OnParseTokenCallback, OnPrintCallback, OnVarCallback};
+use crate::api::options::LanguageOptions;
+use crate::func::native::{
+    OnDebugCallback, OnDefVarCallback, OnParseTokenCallback, OnPrintCallback, OnVarCallback,
+};
 use crate::packages::{Package, StandardPackage};
 use crate::tokenizer::Token;
-use crate::types::dynamic::{map_std_type_name, Union};
+use crate::types::dynamic::Union;
 use crate::{
-    Dynamic, Identifier, ImmutableString, Module, Position, RhaiError, RhaiResult, Shared,
-    StaticVec, ERR,
+    Dynamic, Identifier, ImmutableString, Module, Position, RhaiResult, Shared, StaticVec,
 };
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 use std::{
-    any::type_name,
     collections::{BTreeMap, BTreeSet},
     fmt,
     num::NonZeroU8,
@@ -96,14 +97,12 @@ pub struct Engine {
     /// A collection of all modules loaded into the global namespace of the Engine.
     pub(crate) global_modules: StaticVec<Shared<Module>>,
     /// A collection of all sub-modules directly loaded into the Engine.
+    #[cfg(not(feature = "no_module"))]
     pub(crate) global_sub_modules: BTreeMap<Identifier, Shared<Module>>,
 
     /// A module resolution service.
     #[cfg(not(feature = "no_module"))]
-    pub(crate) module_resolver: Option<Box<dyn crate::ModuleResolver>>,
-
-    /// A map mapping type names to pretty-print names.
-    pub(crate) type_names: BTreeMap<Identifier, Identifier>,
+    pub(crate) module_resolver: Box<dyn crate::ModuleResolver>,
 
     /// An empty [`ImmutableString`] for cloning purposes.
     pub(crate) empty_string: ImmutableString,
@@ -114,29 +113,34 @@ pub struct Engine {
     pub(crate) custom_keywords: BTreeMap<Identifier, Option<Precedence>>,
     /// Custom syntax.
     pub(crate) custom_syntax: BTreeMap<Identifier, Box<CustomSyntax>>,
+    /// Callback closure for filtering variable definition.
+    pub(crate) def_var_filter: Option<Box<OnDefVarCallback>>,
     /// Callback closure for resolving variable access.
-    pub(crate) resolve_var: Option<OnVarCallback>,
+    pub(crate) resolve_var: Option<Box<OnVarCallback>>,
     /// Callback closure to remap tokens during parsing.
     pub(crate) token_mapper: Option<Box<OnParseTokenCallback>>,
 
     /// Callback closure for implementing the `print` command.
-    pub(crate) print: Option<OnPrintCallback>,
+    pub(crate) print: Box<OnPrintCallback>,
     /// Callback closure for implementing the `debug` command.
-    pub(crate) debug: Option<OnDebugCallback>,
+    pub(crate) debug: Box<OnDebugCallback>,
     /// Callback closure for progress reporting.
     #[cfg(not(feature = "unchecked"))]
-    pub(crate) progress: Option<crate::func::native::OnProgressCallback>,
-
-    /// Optimize the [`AST`][crate::AST] after compilation.
-    #[cfg(not(feature = "no_optimize"))]
-    pub(crate) optimization_level: crate::OptimizationLevel,
+    pub(crate) progress: Option<Box<crate::func::native::OnProgressCallback>>,
 
     /// Language options.
-    pub(crate) options: crate::api::options::LanguageOptions,
+    pub(crate) options: LanguageOptions,
 
     /// Max limits.
     #[cfg(not(feature = "unchecked"))]
     pub(crate) limits: crate::api::limits::Limits,
+
+    /// Callback closure for debugging.
+    #[cfg(feature = "debugging")]
+    pub(crate) debugger: Option<(
+        Box<crate::eval::OnDebuggingInit>,
+        Box<crate::eval::OnDebuggerCallback>,
+    )>,
 }
 
 impl fmt::Debug for Engine {
@@ -144,27 +148,21 @@ impl fmt::Debug for Engine {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let mut f = f.debug_struct("Engine");
 
-        f.field("global_modules", &self.global_modules)
-            .field("global_sub_modules", &self.global_sub_modules);
+        f.field("global_modules", &self.global_modules);
 
         #[cfg(not(feature = "no_module"))]
-        f.field("module_resolver", &self.module_resolver.is_some());
+        f.field("global_sub_modules", &self.global_sub_modules);
 
-        f.field("type_names", &self.type_names)
-            .field("disabled_symbols", &self.disabled_symbols)
+        f.field("disabled_symbols", &self.disabled_symbols)
             .field("custom_keywords", &self.custom_keywords)
             .field("custom_syntax", &(!self.custom_syntax.is_empty()))
+            .field("def_var_filter", &self.def_var_filter.is_some())
             .field("resolve_var", &self.resolve_var.is_some())
-            .field("token_mapper", &self.token_mapper.is_some())
-            .field("print", &self.print.is_some())
-            .field("debug", &self.debug.is_some());
+            .field("token_mapper", &self.token_mapper.is_some());
 
         #[cfg(not(feature = "unchecked"))]
         f.field("progress", &self.progress.is_some());
 
-        #[cfg(not(feature = "no_optimize"))]
-        f.field("optimization_level", &self.optimization_level);
-
         f.field("options", &self.options);
 
         #[cfg(not(feature = "unchecked"))]
@@ -215,29 +213,23 @@ impl Engine {
         #[cfg(not(feature = "no_std"))]
         #[cfg(not(target_family = "wasm"))]
         {
-            engine.module_resolver =
-                Some(Box::new(crate::module::resolvers::FileModuleResolver::new()));
+            engine.module_resolver = Box::new(crate::module::resolvers::FileModuleResolver::new());
         }
 
         // default print/debug implementations
         #[cfg(not(feature = "no_std"))]
         #[cfg(not(target_family = "wasm"))]
         {
-            engine.print = Some(Box::new(|s| println!("{}", s)));
-            engine.debug = Some(Box::new(|s, source, pos| {
+            engine.print = Box::new(|s| println!("{}", s));
+            engine.debug = Box::new(|s, source, pos| {
                 if let Some(source) = source {
-                    println!("{}{:?} | {}", source, pos, s);
+                    println!("{} @ {:?} | {}", source, pos, s);
                 } else if pos.is_none() {
                     println!("{}", s);
                 } else {
                     println!("{:?} | {}", pos, s);
                 }
-            }));
-        }
-        #[cfg(any(feature = "no_std", target_family = "wasm"))]
-        {
-            engine.print = None;
-            engine.debug = None;
+            });
         }
 
         engine.register_global_module(StandardPackage::new().as_shared_module());
@@ -253,33 +245,35 @@ impl Engine {
     pub fn new_raw() -> Self {
         let mut engine = Self {
             global_modules: StaticVec::new_const(),
+
+            #[cfg(not(feature = "no_module"))]
             global_sub_modules: BTreeMap::new(),
 
             #[cfg(not(feature = "no_module"))]
-            module_resolver: None,
+            module_resolver: Box::new(crate::module::resolvers::DummyModuleResolver::new()),
 
-            type_names: BTreeMap::new(),
             empty_string: ImmutableString::new(),
             disabled_symbols: BTreeSet::new(),
             custom_keywords: BTreeMap::new(),
             custom_syntax: BTreeMap::new(),
 
+            def_var_filter: None,
             resolve_var: None,
             token_mapper: None,
 
-            print: None,
-            debug: None,
+            print: Box::new(|_| {}),
+            debug: Box::new(|_, _, _| {}),
 
             #[cfg(not(feature = "unchecked"))]
             progress: None,
 
-            #[cfg(not(feature = "no_optimize"))]
-            optimization_level: crate::OptimizationLevel::default(),
-
-            options: crate::api::options::LanguageOptions::new(),
+            options: LanguageOptions::new(),
 
             #[cfg(not(feature = "unchecked"))]
             limits: crate::api::limits::Limits::new(),
+
+            #[cfg(feature = "debugging")]
+            debugger: None,
         };
 
         // Add the global namespace module
@@ -307,7 +301,7 @@ impl Engine {
         match result {
             Ok(ref mut r) => {
                 // Concentrate all empty strings into one instance to save memory
-                if let Dynamic(Union::Str(s, _, _)) = r {
+                if let Dynamic(Union::Str(s, ..)) = r {
                     if s.is_empty() {
                         if !s.ptr_eq(&self.empty_string) {
                             *s = self.const_empty_string();
@@ -324,59 +318,4 @@ impl Engine {
 
         result
     }
-
-    /// Pretty-print a type name.
-    ///
-    /// If a type is registered via [`register_type_with_name`][Engine::register_type_with_name],
-    /// the type name provided for the registration will be used.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the type name is `&mut`.
-    #[inline]
-    #[must_use]
-    pub fn map_type_name<'a>(&'a self, name: &'a str) -> &'a str {
-        self.type_names
-            .get(name)
-            .map(|s| s.as_str())
-            .unwrap_or_else(|| map_std_type_name(name, true))
-    }
-
-    /// Format a type name.
-    ///
-    /// If a type is registered via [`register_type_with_name`][Engine::register_type_with_name],
-    /// the type name provided for the registration will be used.
-    #[cfg(feature = "metadata")]
-    #[inline]
-    #[must_use]
-    pub(crate) fn format_type_name<'a>(&'a self, name: &'a str) -> std::borrow::Cow<'a, str> {
-        if name.starts_with("&mut ") {
-            let x = &name[5..];
-            let r = self.format_type_name(x);
-            return if x != r {
-                format!("&mut {}", r).into()
-            } else {
-                name.into()
-            };
-        }
-
-        self.type_names
-            .get(name)
-            .map(|s| s.as_str())
-            .unwrap_or_else(|| match name {
-                "INT" => return type_name::<crate::INT>(),
-                #[cfg(not(feature = "no_float"))]
-                "FLOAT" => return type_name::<crate::FLOAT>(),
-                _ => map_std_type_name(name, false),
-            })
-            .into()
-    }
-
-    /// Make a `Box<`[`EvalAltResult<ErrorMismatchDataType>`][ERR::ErrorMismatchDataType]`>`.
-    #[inline]
-    #[must_use]
-    pub(crate) fn make_type_mismatch_err<T>(&self, typ: &str, pos: Position) -> RhaiError {
-        ERR::ErrorMismatchDataType(self.map_type_name(type_name::<T>()).into(), typ.into(), pos)
-            .into()
-    }
 }
