@@ -1,12 +1,11 @@
 //! Module defining external-loaded modules for Rhai.
 
-use crate::ast::{FnAccess, Ident};
+use crate::ast::FnAccess;
 use crate::func::{
     shared_take_or_clone, CallableFunction, FnCallArgs, IteratorFn, RegisterNativeFunction,
     SendSync,
 };
-use crate::tokenizer::Token;
-use crate::types::dynamic::Variant;
+use crate::types::{dynamic::Variant, CustomTypesCollection};
 use crate::{
     calc_fn_params_hash, calc_qualified_fn_hash, combine_hashes, Dynamic, Identifier,
     ImmutableString, NativeCallContext, RhaiResultOf, Shared, StaticVec,
@@ -19,17 +18,18 @@ use std::{
     collections::{BTreeMap, BTreeSet},
     fmt,
     iter::{empty, once},
-    num::NonZeroUsize,
-    ops::{Add, AddAssign, Deref, DerefMut},
+    ops::{Add, AddAssign},
 };
 
 /// A type representing the namespace of a function.
-#[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum FnNamespace {
+    /// Module namespace only.
+    ///
+    /// Ignored under `no_module`.
+    Internal,
     /// Expose to global namespace.
     Global,
-    /// Module namespace only.
-    Internal,
 }
 
 /// A type containing all metadata for a registered function.
@@ -51,7 +51,7 @@ pub struct FnMetadata {
     pub return_type: Identifier,
     /// Comments.
     #[cfg(feature = "metadata")]
-    pub comments: Option<Box<[Box<str>]>>,
+    pub comments: Box<[Box<str>]>,
 }
 
 impl PartialOrd for FnMetadata {
@@ -151,7 +151,7 @@ impl FuncInfo {
             ty => ty.into(),
         }
     }
-    /// Generate a signature of the function.
+    /// _(metadata)_ Generate a signature of the function.
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     #[must_use]
@@ -232,6 +232,8 @@ pub struct Module {
     pub(crate) internal: bool,
     /// Is this module part of a standard library?
     pub(crate) standard: bool,
+    /// Custom types.
+    custom_types: CustomTypesCollection,
     /// Sub-modules.
     modules: BTreeMap<Identifier, Shared<Module>>,
     /// [`Module`] variables.
@@ -244,9 +246,9 @@ pub struct Module {
     /// including those in sub-modules.
     all_functions: BTreeMap<u64, Shared<CallableFunction>>,
     /// Iterator functions, keyed by the type producing the iterator.
-    type_iterators: BTreeMap<TypeId, IteratorFn>,
+    type_iterators: BTreeMap<TypeId, Shared<IteratorFn>>,
     /// Flattened collection of iterator functions, including those in sub-modules.
-    all_type_iterators: BTreeMap<TypeId, IteratorFn>,
+    all_type_iterators: BTreeMap<TypeId, Shared<IteratorFn>>,
     /// Is the [`Module`] indexed?
     indexed: bool,
     /// Does the [`Module`] contain indexed functions that have been exposed to the global namespace?
@@ -339,6 +341,7 @@ impl Module {
             id: Identifier::new_const(),
             internal: false,
             standard: false,
+            custom_types: CustomTypesCollection::new(),
             modules: BTreeMap::new(),
             variables: BTreeMap::new(),
             all_variables: BTreeMap::new(),
@@ -413,6 +416,26 @@ impl Module {
         self
     }
 
+    /// Map a custom type to a friendly display name.
+    #[inline(always)]
+    pub fn set_custom_type<T>(&mut self, name: &str) {
+        self.custom_types.add_type::<T>(name)
+    }
+    /// Map a custom type to a friendly display name.
+    #[inline(always)]
+    pub fn set_custom_type_raw(
+        &mut self,
+        type_name: impl Into<Identifier>,
+        name: impl Into<Identifier>,
+    ) {
+        self.custom_types.add(type_name, name)
+    }
+    /// Get the display name of a registered custom type.
+    #[inline(always)]
+    pub fn get_custom_type(&self, key: &str) -> Option<&str> {
+        self.custom_types.get(key)
+    }
+
     /// Is the [`Module`] empty?
     ///
     /// # Example
@@ -462,7 +485,7 @@ impl Module {
         self.indexed
     }
 
-    /// Generate signatures for all the non-private functions in the [`Module`].
+    /// _(metadata)_ Generate signatures for all the non-private functions in the [`Module`].
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     #[inline]
@@ -488,7 +511,11 @@ impl Module {
     #[inline(always)]
     #[must_use]
     pub fn contains_var(&self, name: &str) -> bool {
-        self.variables.contains_key(name)
+        if !self.variables.is_empty() {
+            self.variables.contains_key(name)
+        } else {
+            false
+        }
     }
 
     /// Get the value of a [`Module`] variable.
@@ -520,7 +547,11 @@ impl Module {
     #[inline(always)]
     #[must_use]
     pub fn get_var(&self, name: &str) -> Option<Dynamic> {
-        self.variables.get(name).cloned()
+        if !self.variables.is_empty() {
+            self.variables.get(name).cloned()
+        } else {
+            None
+        }
     }
 
     /// Set a variable into the [`Module`].
@@ -552,14 +583,15 @@ impl Module {
         self
     }
 
-    /// Get a reference to a namespace-qualified variable.
-    /// Name and Position in [`EvalAltResult`] are [`None`] and [`NONE`][Position::NONE] and must be set afterwards.
+    /// Get a namespace-qualified [`Module`] variable as a [`Dynamic`].
     #[cfg(not(feature = "no_module"))]
     #[inline]
-    pub(crate) fn get_qualified_var(&self, hash_var: u64) -> RhaiResultOf<&Dynamic> {
-        self.all_variables.get(&hash_var).ok_or_else(|| {
-            crate::ERR::ErrorVariableNotFound(String::new(), crate::Position::NONE).into()
-        })
+    pub(crate) fn get_qualified_var(&self, hash_var: u64) -> Option<Dynamic> {
+        if !self.all_variables.is_empty() {
+            self.all_variables.get(&hash_var).cloned()
+        } else {
+            None
+        }
     }
 
     /// Set a script-defined function into the [`Module`].
@@ -588,7 +620,7 @@ impl Module {
                     #[cfg(feature = "metadata")]
                     return_type: "".into(),
                     #[cfg(feature = "metadata")]
-                    comments: None,
+                    comments: Box::default(),
                 },
                 func: Into::<CallableFunction>::into(fn_def).into(),
                 param_types: StaticVec::new_const(),
@@ -610,14 +642,14 @@ impl Module {
         name: impl AsRef<str>,
         num_params: usize,
     ) -> Option<&Shared<crate::ast::ScriptFnDef>> {
-        if self.functions.is_empty() {
-            None
-        } else {
+        if !self.functions.is_empty() {
             let name = name.as_ref();
 
             self.iter_fn()
                 .find(|f| f.metadata.params == num_params && f.metadata.name == name)
                 .and_then(|f| f.func.get_script_fn_def())
+        } else {
+            None
         }
     }
 
@@ -656,7 +688,11 @@ impl Module {
     #[inline(always)]
     #[must_use]
     pub fn contains_sub_module(&self, name: &str) -> bool {
-        self.modules.contains_key(name)
+        if !self.modules.is_empty() {
+            self.modules.contains_key(name)
+        } else {
+            false
+        }
     }
 
     /// Get a sub-module in the [`Module`].
@@ -673,7 +709,11 @@ impl Module {
     #[inline]
     #[must_use]
     pub fn get_sub_module(&self, name: &str) -> Option<&Module> {
-        self.modules.get(name).map(|m| m.as_ref())
+        if !self.modules.is_empty() {
+            self.modules.get(name).map(|m| m.as_ref())
+        } else {
+            None
+        }
     }
 
     /// Set a sub-module into the [`Module`].
@@ -716,7 +756,11 @@ impl Module {
     #[inline(always)]
     #[must_use]
     pub fn contains_fn(&self, hash_fn: u64) -> bool {
-        self.functions.contains_key(&hash_fn)
+        if !self.functions.is_empty() {
+            self.functions.contains_key(&hash_fn)
+        } else {
+            false
+        }
     }
 
     /// _(metadata)_ Update the metadata (parameter names/types and return type) of a registered function.
@@ -759,8 +803,7 @@ impl Module {
         self
     }
 
-    /// _(metadata)_ Update the metadata (parameter names/types, return type and doc-comments) of a
-    /// registered function.
+    /// _(metadata)_ Update the metadata (parameter names/types, return type and doc-comments) of a registered function.
     /// Exported under the `metadata` feature only.
     ///
     /// The [`u64`] hash is returned by the [`set_native_fn`][Module::set_native_fn] call.
@@ -796,7 +839,7 @@ impl Module {
 
         if !comments.is_empty() {
             let f = self.functions.get_mut(&hash_fn).unwrap();
-            f.metadata.comments = Some(comments.iter().map(|s| s.as_ref().into()).collect());
+            f.metadata.comments = comments.iter().map(|s| s.as_ref().into()).collect();
         }
 
         self
@@ -903,7 +946,7 @@ impl Module {
                     #[cfg(feature = "metadata")]
                     return_type: return_type_name,
                     #[cfg(feature = "metadata")]
-                    comments: None,
+                    comments: Box::default(),
                 },
                 func: func.into(),
                 param_types,
@@ -961,7 +1004,7 @@ impl Module {
 
         if !comments.is_empty() {
             let f = self.functions.get_mut(&hash).unwrap();
-            f.metadata.comments = Some(comments.iter().map(|s| s.as_ref().into()).collect());
+            f.metadata.comments = comments.iter().map(|s| s.as_ref().into()).collect();
         }
 
         hash
@@ -1358,7 +1401,11 @@ impl Module {
     #[inline]
     #[must_use]
     pub(crate) fn get_fn(&self, hash_fn: u64) -> Option<&CallableFunction> {
-        self.functions.get(&hash_fn).map(|f| f.func.as_ref())
+        if !self.functions.is_empty() {
+            self.functions.get(&hash_fn).map(|f| f.func.as_ref())
+        } else {
+            None
+        }
     }
 
     /// Does the particular namespace-qualified function exist in the [`Module`]?
@@ -1367,18 +1414,27 @@ impl Module {
     #[inline(always)]
     #[must_use]
     pub fn contains_qualified_fn(&self, hash_fn: u64) -> bool {
-        self.all_functions.contains_key(&hash_fn)
+        if !self.all_functions.is_empty() {
+            self.all_functions.contains_key(&hash_fn)
+        } else {
+            false
+        }
     }
 
     /// Get a namespace-qualified function.
     ///
     /// The [`u64`] hash is calculated by [`build_index`][Module::build_index].
+    #[cfg(not(feature = "no_module"))]
     #[inline]
     #[must_use]
     pub(crate) fn get_qualified_fn(&self, hash_qualified_fn: u64) -> Option<&CallableFunction> {
-        self.all_functions
-            .get(&hash_qualified_fn)
-            .map(|f| f.as_ref())
+        if !self.all_functions.is_empty() {
+            self.all_functions
+                .get(&hash_qualified_fn)
+                .map(|f| f.as_ref())
+        } else {
+            None
+        }
     }
 
     /// Combine another [`Module`] into this [`Module`].
@@ -1402,9 +1458,9 @@ impl Module {
     /// Sub-modules are flattened onto the root [`Module`], with higher level overriding lower level.
     #[inline]
     pub fn combine_flatten(&mut self, other: Self) -> &mut Self {
-        other.modules.into_iter().for_each(|(_, m)| {
+        for (.., m) in other.modules.into_iter() {
             self.combine_flatten(shared_take_or_clone(m));
-        });
+        }
         self.variables.extend(other.variables.into_iter());
         self.functions.extend(other.functions.into_iter());
         self.type_iterators.extend(other.type_iterators.into_iter());
@@ -1420,22 +1476,22 @@ impl Module {
     /// Only items not existing in this [`Module`] are added.
     #[inline]
     pub fn fill_with(&mut self, other: &Self) -> &mut Self {
-        other.modules.iter().for_each(|(k, v)| {
+        for (k, v) in &other.modules {
             if !self.modules.contains_key(k) {
                 self.modules.insert(k.clone(), v.clone());
             }
-        });
-        other.variables.iter().for_each(|(k, v)| {
+        }
+        for (k, v) in &other.variables {
             if !self.variables.contains_key(k) {
                 self.variables.insert(k.clone(), v.clone());
             }
-        });
-        other.functions.iter().for_each(|(&k, v)| {
+        }
+        for (&k, v) in &other.functions {
             self.functions.entry(k).or_insert_with(|| v.clone());
-        });
-        other.type_iterators.iter().for_each(|(&k, &v)| {
-            self.type_iterators.entry(k).or_insert(v);
-        });
+        }
+        for (&k, v) in &other.type_iterators {
+            self.type_iterators.entry(k).or_insert_with(|| v.clone());
+        }
         self.all_functions.clear();
         self.all_variables.clear();
         self.all_type_iterators.clear();
@@ -1456,12 +1512,11 @@ impl Module {
         other: &Self,
         _filter: impl Fn(FnNamespace, FnAccess, bool, &str, usize) -> bool + Copy,
     ) -> &mut Self {
-        #[cfg(not(feature = "no_function"))]
-        other.modules.iter().for_each(|(k, v)| {
+        for (k, v) in &other.modules {
             let mut m = Self::new();
             m.merge_filtered(v, _filter);
             self.set_sub_module(k.clone(), m);
-        });
+        }
         #[cfg(feature = "no_function")]
         self.modules
             .extend(other.modules.iter().map(|(k, v)| (k.clone(), v.clone())));
@@ -1472,7 +1527,7 @@ impl Module {
             other
                 .functions
                 .iter()
-                .filter(|&(_, f)| {
+                .filter(|&(.., f)| {
                     _filter(
                         f.metadata.namespace,
                         f.metadata.access,
@@ -1484,7 +1539,8 @@ impl Module {
                 .map(|(&k, v)| (k, v.clone())),
         );
 
-        self.type_iterators.extend(other.type_iterators.iter());
+        self.type_iterators
+            .extend(other.type_iterators.iter().map(|(&k, v)| (k, v.clone())));
         self.all_functions.clear();
         self.all_variables.clear();
         self.all_type_iterators.clear();
@@ -1502,7 +1558,7 @@ impl Module {
     ) -> &mut Self {
         self.functions = std::mem::take(&mut self.functions)
             .into_iter()
-            .filter(|(_, f)| {
+            .filter(|(.., f)| {
                 if f.func.is_script() {
                     filter(
                         f.metadata.namespace,
@@ -1658,63 +1714,110 @@ impl Module {
     /// # }
     /// ```
     #[cfg(not(feature = "no_module"))]
+    #[inline(always)]
     pub fn eval_ast_as_new(
         scope: crate::Scope,
         ast: &crate::AST,
         engine: &crate::Engine,
+    ) -> RhaiResultOf<Self> {
+        let global = &mut crate::eval::GlobalRuntimeState::new(engine);
+
+        Self::eval_ast_as_new_raw(engine, scope, global, ast)
+    }
+    /// Create a new [`Module`] by evaluating an [`AST`][crate::AST].
+    ///
+    /// The entire [`AST`][crate::AST] is encapsulated into each function, allowing functions to
+    /// cross-call each other.
+    ///
+    /// Functions in the global namespace, plus all functions defined in the [`Module`], are
+    /// _merged_ into a _unified_ namespace before each call.
+    ///
+    /// Therefore, all functions will be found.
+    #[cfg(not(feature = "no_module"))]
+    pub(crate) fn eval_ast_as_new_raw(
+        engine: &crate::Engine,
+        scope: crate::Scope,
+        global: &mut crate::eval::GlobalRuntimeState,
+        ast: &crate::AST,
     ) -> RhaiResultOf<Self> {
         let mut scope = scope;
-        let mut global = crate::eval::GlobalRuntimeState::new();
-        let orig_mods_len = global.num_imports();
+
+        // Save global state
+        let orig_imports_len = global.num_imports();
+        let orig_source = global.source.clone();
+        #[cfg(not(feature = "no_function"))]
+        let orig_constants = std::mem::take(&mut global.constants);
 
         // Run the script
-        engine.eval_ast_with_scope_raw(&mut scope, &mut global, &ast, 0)?;
+        let result = engine.eval_ast_with_scope_raw(&mut scope, global, &ast, 0);
 
         // Create new module
-        let mut module =
-            scope
-                .into_iter()
-                .fold(Module::new(), |mut module, (_, value, mut aliases)| {
-                    // Variables with an alias left in the scope become module variables
-                    match aliases.len() {
-                        0 => (),
-                        1 => {
-                            let alias = aliases.pop().unwrap();
-                            module.set_var(alias, value);
-                        }
-                        _ => {
-                            let last_alias = aliases.pop().unwrap();
-                            aliases.into_iter().for_each(|alias| {
-                                module.set_var(alias, value.clone());
-                            });
-                            // Avoid cloning the last value
-                            module.set_var(last_alias, value);
-                        }
-                    }
-                    module
+        let mut module = Module::new();
+
+        // Extra modules left become sub-modules
+        let mut imports = StaticVec::new_const();
+
+        if result.is_ok() {
+            global
+                .scan_imports_raw()
+                .skip(orig_imports_len)
+                .for_each(|(k, m)| {
+                    imports.push((k.clone(), m.clone()));
+                    module.set_sub_module(k.clone(), m.clone());
                 });
+        }
 
-        // Extra modules left in the scope become sub-modules
+        // Restore global state
         #[cfg(not(feature = "no_function"))]
-        let mut func_global = None;
+        let constants = std::mem::replace(&mut global.constants, orig_constants);
+        global.truncate_imports(orig_imports_len);
+        global.source = orig_source;
 
-        global.into_iter().skip(orig_mods_len).for_each(|kv| {
+        result?;
+
+        // Variables with an alias left in the scope become module variables
+        for (_name, value, mut aliases) in scope {
+            // It is an error to export function pointers that refer to encapsulated local functions
             #[cfg(not(feature = "no_function"))]
-            if func_global.is_none() {
-                func_global = Some(StaticVec::new());
+            if let Some(fn_ptr) = value.downcast_ref::<crate::FnPtr>() {
+                if ast.iter_fn_def().any(|f| f.name == fn_ptr.fn_name()) {
+                    return Err(crate::ERR::ErrorMismatchDataType(
+                        "".to_string(),
+                        if fn_ptr.is_anonymous() {
+                            format!("cannot export closure in variable {}", _name)
+                        } else {
+                            format!(
+                                "cannot export function pointer to local function '{}' in variable {}",
+                                fn_ptr.fn_name(),
+                                _name
+                            )
+                        },
+                        crate::Position::NONE,
+                    )
+                    .into());
+                }
             }
-            #[cfg(not(feature = "no_function"))]
-            func_global.as_mut().expect("`Some`").push(kv.clone());
 
-            module.set_sub_module(kv.0, kv.1);
-        });
-
-        #[cfg(not(feature = "no_function"))]
-        let func_global = func_global.map(|v| v.into_boxed_slice());
+            match aliases.len() {
+                0 => (),
+                1 => {
+                    let alias = aliases.pop().unwrap();
+                    module.set_var(alias, value);
+                }
+                _ => {
+                    let last_alias = aliases.pop().unwrap();
+                    for alias in aliases {
+                        module.set_var(alias, value.clone());
+                    }
+                    // Avoid cloning the last value
+                    module.set_var(last_alias, value);
+                }
+            }
+        }
 
         // Non-private functions defined become module functions
         #[cfg(not(feature = "no_function"))]
-        if ast.has_functions() {
+        {
             ast.shared_lib()
                 .iter_fn()
                 .filter(|&f| match f.metadata.access {
@@ -1723,15 +1826,20 @@ impl Module {
                 })
                 .filter(|&f| f.func.is_script())
                 .for_each(|f| {
-                    // Encapsulate AST environment
                     let mut func = f
                         .func
                         .get_script_fn_def()
                         .expect("script-defined function")
                         .as_ref()
                         .clone();
-                    func.lib = Some(ast.shared_lib().clone());
-                    func.global = func_global.clone();
+
+                    // Encapsulate AST environment
+                    func.environ = Some(crate::ast::EncapsulatedEnviron {
+                        lib: ast.shared_lib().clone(),
+                        imports: imports.clone().into_boxed_slice(),
+                        constants: constants.clone(),
+                    });
+
                     module.set_script_fn(func);
                 });
         }
@@ -1765,33 +1873,33 @@ impl Module {
             path: &mut Vec<&'a str>,
             variables: &mut BTreeMap<u64, Dynamic>,
             functions: &mut BTreeMap<u64, Shared<CallableFunction>>,
-            type_iterators: &mut BTreeMap<TypeId, IteratorFn>,
+            type_iterators: &mut BTreeMap<TypeId, Shared<IteratorFn>>,
         ) -> bool {
             let mut contains_indexed_global_functions = false;
 
-            module.modules.iter().for_each(|(name, m)| {
+            for (name, m) in &module.modules {
                 // Index all the sub-modules first.
                 path.push(name);
                 if index_module(m, path, variables, functions, type_iterators) {
                     contains_indexed_global_functions = true;
                 }
                 path.pop();
-            });
+            }
 
             // Index all variables
-            module.variables.iter().for_each(|(var_name, value)| {
+            for (var_name, value) in &module.variables {
                 let hash_var = crate::calc_qualified_var_hash(path.iter().copied(), var_name);
                 variables.insert(hash_var, value.clone());
-            });
+            }
 
             // Index type iterators
-            module.type_iterators.iter().for_each(|(&type_id, func)| {
-                type_iterators.insert(type_id, *func);
+            for (&type_id, func) in &module.type_iterators {
+                type_iterators.insert(type_id, func.clone());
                 contains_indexed_global_functions = true;
-            });
+            }
 
             // Index all Rust functions
-            module.functions.iter().for_each(|(&hash, f)| {
+            for (&hash, f) in &module.functions {
                 match f.metadata.namespace {
                     FnNamespace::Global => {
                         // Flatten all functions with global namespace
@@ -1802,7 +1910,7 @@ impl Module {
                 }
                 match f.metadata.access {
                     FnAccess::Public => (),
-                    FnAccess::Private => return, // Do not index private functions
+                    FnAccess::Private => continue, // Do not index private functions
                 }
 
                 if !f.func.is_script() {
@@ -1820,7 +1928,7 @@ impl Module {
                     );
                     functions.insert(hash_qualified_script, f.func.clone());
                 }
-            });
+            }
 
             contains_indexed_global_functions
         }
@@ -1854,21 +1962,52 @@ impl Module {
     #[inline(always)]
     #[must_use]
     pub fn contains_qualified_iter(&self, id: TypeId) -> bool {
-        self.all_type_iterators.contains_key(&id)
+        if !self.all_type_iterators.is_empty() {
+            self.all_type_iterators.contains_key(&id)
+        } else {
+            false
+        }
     }
 
     /// Does a type iterator exist in the module?
     #[inline(always)]
     #[must_use]
     pub fn contains_iter(&self, id: TypeId) -> bool {
-        self.type_iterators.contains_key(&id)
+        if !self.type_iterators.is_empty() {
+            self.type_iterators.contains_key(&id)
+        } else {
+            false
+        }
     }
 
     /// Set a type iterator into the [`Module`].
+    #[cfg(not(feature = "sync"))]
     #[inline]
-    pub fn set_iter(&mut self, type_id: TypeId, func: IteratorFn) -> &mut Self {
+    pub fn set_iter(
+        &mut self,
+        type_id: TypeId,
+        func: impl Fn(Dynamic) -> Box<dyn Iterator<Item = Dynamic>> + 'static,
+    ) -> &mut Self {
+        let func = Shared::new(func);
         if self.indexed {
-            self.all_type_iterators.insert(type_id, func);
+            self.all_type_iterators.insert(type_id, func.clone());
+            self.contains_indexed_global_functions = true;
+        }
+        self.type_iterators.insert(type_id, func);
+        self
+    }
+
+    /// Set a type iterator into the [`Module`].
+    #[cfg(feature = "sync")]
+    #[inline]
+    pub fn set_iter(
+        &mut self,
+        type_id: TypeId,
+        func: impl Fn(Dynamic) -> Box<dyn Iterator<Item = Dynamic>> + SendSync + 'static,
+    ) -> &mut Self {
+        let func = Shared::new(func);
+        if self.indexed {
+            self.all_type_iterators.insert(type_id, func.clone());
             self.contains_indexed_global_functions = true;
         }
         self.type_iterators.insert(type_id, func);
@@ -1900,126 +2039,32 @@ impl Module {
     }
 
     /// Get the specified type iterator.
+    #[cfg(not(feature = "no_module"))]
     #[inline]
     #[must_use]
-    pub(crate) fn get_qualified_iter(&self, id: TypeId) -> Option<IteratorFn> {
-        self.all_type_iterators.get(&id).cloned()
+    pub(crate) fn get_qualified_iter(&self, id: TypeId) -> Option<&IteratorFn> {
+        if !self.all_type_iterators.is_empty() {
+            self.all_type_iterators.get(&id).map(|f| f.as_ref())
+        } else {
+            None
+        }
     }
 
     /// Get the specified type iterator.
     #[inline]
     #[must_use]
-    pub(crate) fn get_iter(&self, id: TypeId) -> Option<IteratorFn> {
-        self.type_iterators.get(&id).cloned()
-    }
-}
-
-/// _(internals)_ A chain of [module][Module] names to namespace-qualify a variable or function
-/// call. Exported under the `internals` feature only.
-///
-/// A [`u64`] offset to the current [stack of imported modules][crate::GlobalRuntimeState] is
-/// cached for quick search purposes.
-///
-/// A [`StaticVec`] is used because the vast majority of namespace-qualified access contains only
-/// one level, and it is wasteful to always allocate a [`Vec`] with one element.
-#[derive(Clone, Eq, PartialEq, Default, Hash)]
-pub struct Namespace {
-    index: Option<NonZeroUsize>,
-    path: StaticVec<Ident>,
-}
-
-impl fmt::Debug for Namespace {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        if let Some(index) = self.index {
-            write!(f, "{} -> ", index)?;
-        }
-
-        f.write_str(
-            &self
-                .path
-                .iter()
-                .map(|Ident { name, .. }| name.as_str())
-                .collect::<StaticVec<_>>()
-                .join(Token::DoubleColon.literal_syntax()),
-        )
-    }
-}
-
-impl fmt::Display for Namespace {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.write_str(
-            &self
-                .path
-                .iter()
-                .map(|Ident { name, .. }| name.as_str())
-                .collect::<StaticVec<_>>()
-                .join(Token::DoubleColon.literal_syntax()),
-        )
-    }
-}
-
-impl Deref for Namespace {
-    type Target = StaticVec<Ident>;
-
-    #[inline(always)]
-    fn deref(&self) -> &Self::Target {
-        &self.path
-    }
-}
-
-impl DerefMut for Namespace {
-    #[inline(always)]
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.path
-    }
-}
-
-impl From<Vec<Ident>> for Namespace {
-    #[inline(always)]
-    fn from(mut path: Vec<Ident>) -> Self {
-        path.shrink_to_fit();
-        Self {
-            index: None,
-            path: path.into(),
+    pub(crate) fn get_iter(&self, id: TypeId) -> Option<&IteratorFn> {
+        if !self.type_iterators.is_empty() {
+            self.type_iterators.get(&id).map(|f| f.as_ref())
+        } else {
+            None
         }
     }
 }
 
-impl From<StaticVec<Ident>> for Namespace {
-    #[inline(always)]
-    fn from(mut path: StaticVec<Ident>) -> Self {
-        path.shrink_to_fit();
-        Self { index: None, path }
-    }
-}
-
-impl Namespace {
-    /// Create a new [`Namespace`].
-    #[inline(always)]
-    #[must_use]
-    pub const fn new() -> Self {
-        Self {
-            index: None,
-            path: StaticVec::new_const(),
-        }
-    }
-    /// Get the [`Scope`][crate::Scope] index offset.
-    #[inline(always)]
-    #[must_use]
-    pub(crate) const fn index(&self) -> Option<NonZeroUsize> {
-        self.index
-    }
-    /// Set the [`Scope`][crate::Scope] index offset.
-    #[cfg(not(feature = "no_module"))]
-    #[inline(always)]
-    pub(crate) fn set_index(&mut self, index: Option<NonZeroUsize>) {
-        self.index = index
-    }
-}
-
-#[cfg(not(feature = "no_module"))]
-pub use resolvers::ModuleResolver;
-
 /// Module containing all built-in [module resolvers][ModuleResolver].
 #[cfg(not(feature = "no_module"))]
 pub mod resolvers;
+
+#[cfg(not(feature = "no_module"))]
+pub use resolvers::ModuleResolver;
