@@ -11,7 +11,7 @@ use crate::Array;
 
 def_package! {
     /// Package of basic object map utilities.
-    crate::BasicMapPackage => |lib| {
+    pub BasicMapPackage(lib) {
         lib.standard = true;
 
         combine_with_exported_module!(lib, "map", map_functions);
