@@ -1,5 +1,3 @@
-#![allow(non_snake_case)]
-
 use crate::plugin::*;
 use crate::{def_package, Dynamic, ExclusiveRange, InclusiveRange, RhaiResultOf, StaticVec, INT};
 #[cfg(feature = "no_std")]
@@ -8,9 +6,12 @@ use std::{any::TypeId, mem};
 
 use super::string_basic::{print_with_func, FUNC_TO_STRING};
 
+#[cfg(not(feature = "no_index"))]
+use crate::Blob;
+
 def_package! {
     /// Package of additional string utilities over [`BasicStringPackage`][super::BasicStringPackage]
-    crate::MoreStringPackage => |lib| {
+    pub MoreStringPackage(lib) {
         lib.standard = true;
 
         combine_with_exported_module!(lib, "string", string_functions);
@@ -21,20 +22,28 @@ def_package! {
 mod string_functions {
     use crate::{ImmutableString, SmartString};
 
-    #[rhai_fn(name = "+", name = "append")]
+    #[rhai_fn(name = "+", pure)]
     pub fn add_append(
         ctx: NativeCallContext,
-        string: ImmutableString,
+        string: &mut ImmutableString,
         mut item: Dynamic,
     ) -> ImmutableString {
         let s = print_with_func(FUNC_TO_STRING, &ctx, &mut item);
 
         if s.is_empty() {
-            string
+            string.clone()
         } else {
             format!("{}{}", string, s).into()
         }
     }
+    #[rhai_fn(name = "+=", name = "append")]
+    pub fn add(ctx: NativeCallContext, string: &mut ImmutableString, mut item: Dynamic) {
+        let s = print_with_func(FUNC_TO_STRING, &ctx, &mut item);
+
+        if !s.is_empty() {
+            *string = format!("{}{}", string, s).into();
+        }
+    }
     #[rhai_fn(name = "+", pure)]
     pub fn add_prepend(
         ctx: NativeCallContext,
@@ -50,20 +59,22 @@ mod string_functions {
         s
     }
 
-    #[rhai_fn(name = "+", name = "append")]
-    pub fn add_append_str(string1: ImmutableString, string2: ImmutableString) -> ImmutableString {
-        string1 + string2
+    // The following are needed in order to override the generic versions with `Dynamic` parameters.
+
+    #[rhai_fn(name = "+", pure)]
+    pub fn add_append_str(string1: &mut ImmutableString, string2: &str) -> ImmutableString {
+        &*string1 + string2
     }
-    #[rhai_fn(name = "+", name = "append")]
-    pub fn add_append_char(string: ImmutableString, character: char) -> ImmutableString {
-        string + character
+    #[rhai_fn(name = "+", pure)]
+    pub fn add_append_char(string: &mut ImmutableString, character: char) -> ImmutableString {
+        &*string + character
     }
     #[rhai_fn(name = "+")]
-    pub fn add_prepend_char(character: char, string: ImmutableString) -> ImmutableString {
+    pub fn add_prepend_char(character: char, string: &str) -> ImmutableString {
         format!("{}{}", character, string).into()
     }
 
-    #[rhai_fn(name = "+", name = "append")]
+    #[rhai_fn(name = "+")]
     pub fn add_append_unit(string: ImmutableString, item: ()) -> ImmutableString {
         let _item = item;
         string
@@ -73,6 +84,30 @@ mod string_functions {
         string
     }
 
+    #[cfg(not(feature = "no_index"))]
+    pub mod blob_functions {
+        #[rhai_fn(name = "+", pure)]
+        pub fn add_append_blob(string: &mut ImmutableString, utf8: Blob) -> ImmutableString {
+            if utf8.is_empty() {
+                string.clone()
+            } else if string.is_empty() {
+                String::from_utf8_lossy(&utf8).into_owned().into()
+            } else {
+                let mut s = crate::SmartString::from(string.as_str());
+                s.push_str(&String::from_utf8_lossy(&utf8));
+                s.into()
+            }
+        }
+        #[rhai_fn(name = "append")]
+        pub fn add_blob(string: &mut ImmutableString, utf8: Blob) {
+            let mut s = crate::SmartString::from(string.as_str());
+            if !utf8.is_empty() {
+                s.push_str(&String::from_utf8_lossy(&utf8));
+                *string = s.into();
+            }
+        }
+    }
+
     /// Return the length of the string, in number of characters.
     ///
     /// # Example
@@ -107,6 +142,25 @@ mod string_functions {
             string.len() as INT
         }
     }
+    /// Convert the string into an UTF-8 encoded byte-stream as a BLOB.
+    ///
+    /// # Example
+    ///
+    /// ```rhai
+    /// let text = "朝には紅顔ありて夕べには白骨となる";
+    ///
+    /// let bytes = text.to_blob();
+    ///
+    /// print(bytes.len());     // prints 51
+    /// ```
+    #[cfg(not(feature = "no_index"))]
+    pub fn to_blob(string: &str) -> crate::Blob {
+        if string.is_empty() {
+            crate::Blob::new()
+        } else {
+            string.as_bytes().into()
+        }
+    }
     /// Remove all occurrences of a sub-string from the string.
     ///
     /// # Example
@@ -118,7 +172,7 @@ mod string_functions {
     ///
     /// print(text);        // prints ", world! , foobar!"
     /// ```
-    pub fn remove(string: &mut ImmutableString, sub_string: ImmutableString) {
+    pub fn remove(string: &mut ImmutableString, sub_string: &str) {
         *string -= sub_string;
     }
     /// Remove all occurrences of a character from the string.
@@ -139,7 +193,11 @@ mod string_functions {
     /// Clear the string, making it empty.
     pub fn clear(string: &mut ImmutableString) {
         if !string.is_empty() {
-            string.make_mut().clear();
+            if let Some(s) = string.get_mut() {
+                s.clear();
+            } else {
+                *string = ImmutableString::new();
+            }
         }
     }
     /// Cut off the string at the specified number of characters.
@@ -166,8 +224,8 @@ mod string_functions {
             let copy = string.make_mut();
             copy.clear();
             copy.extend(chars.into_iter().take(len as usize));
-        } else if !string.is_empty() {
-            string.make_mut().clear();
+        } else {
+            clear(string);
         }
     }
     /// Remove whitespace characters from both ends of the string.
@@ -177,13 +235,23 @@ mod string_functions {
     /// ```rhai
     /// let text = "   hello     ";
     ///
-    /// print(text.trim());     // prints "hello"
+    /// text.trim();
+    ///
+    /// print(text);    // prints "hello"
     /// ```
     pub fn trim(string: &mut ImmutableString) {
-        let trimmed = string.trim();
+        if let Some(s) = string.get_mut() {
+            let trimmed = s.trim();
 
-        if trimmed.len() < string.len() {
-            *string = trimmed.to_string().into();
+            if trimmed != s {
+                *s = trimmed.into();
+            }
+        } else {
+            let trimmed = string.trim();
+
+            if trimmed != string {
+                *string = trimmed.into();
+            }
         }
     }
     /// Remove the last character from the string and return it.
@@ -209,7 +277,7 @@ mod string_functions {
             }
         }
     }
-    /// Remove the a specified number of characters from the end of the string and return it as a
+    /// Remove a specified number of characters from the end of the string and return it as a
     /// new string.
     ///
     /// * If `len` ≤ 0, the string is not modified and an empty string is returned.
@@ -257,9 +325,10 @@ mod string_functions {
     ///
     /// print(text);                // prints "hello, world!"
     /// ```
-    pub fn to_upper(string: ImmutableString) -> ImmutableString {
-        if string.is_empty() {
-            string
+    #[rhai_fn(pure)]
+    pub fn to_upper(string: &mut ImmutableString) -> ImmutableString {
+        if string.chars().all(char::is_uppercase) {
+            string.clone()
         } else {
             string.to_uppercase().into()
         }
@@ -276,7 +345,7 @@ mod string_functions {
     /// print(text);        // prints "HELLO, WORLD!";
     /// ```
     pub fn make_upper(string: &mut ImmutableString) {
-        if !string.is_empty() {
+        if !string.is_empty() && string.chars().any(|ch| !ch.is_uppercase()) {
             *string = string.to_uppercase().into();
         }
     }
@@ -291,9 +360,10 @@ mod string_functions {
     ///
     /// print(text);                // prints "HELLO, WORLD!"
     /// ```
-    pub fn to_lower(string: ImmutableString) -> ImmutableString {
-        if string.is_empty() {
-            string
+    #[rhai_fn(pure)]
+    pub fn to_lower(string: &mut ImmutableString) -> ImmutableString {
+        if string.is_empty() || string.chars().all(char::is_lowercase) {
+            string.clone()
         } else {
             string.to_lowercase().into()
         }
@@ -310,7 +380,7 @@ mod string_functions {
     /// print(text);        // prints "hello, world!";
     /// ```
     pub fn make_lower(string: &mut ImmutableString) {
-        if !string.is_empty() {
+        if string.chars().any(|ch| !ch.is_lowercase()) {
             *string = string.to_lowercase().into();
         }
     }
@@ -388,6 +458,35 @@ mod string_functions {
         *character = to_lower_char(*character)
     }
 
+    /// Return `true` if the string starts with a specified string.
+    ///
+    /// # Example
+    ///
+    /// ```rhai
+    /// let text = "hello, world!";
+    ///
+    /// print(text.starts_with("hello"));   // prints true
+    ///
+    /// print(text.starts_with("world"));   // prints false
+    /// ```
+    pub fn starts_with(string: &str, match_string: &str) -> bool {
+        string.starts_with(match_string)
+    }
+    /// Return `true` if the string ends with a specified string.
+    ///
+    /// # Example
+    ///
+    /// ```rhai
+    /// let text = "hello, world!";
+    ///
+    /// print(text.ends_with("world!"));    // prints true
+    ///
+    /// print(text.ends_with("hello"));     // prints false
+    /// ```
+    pub fn ends_with(string: &str, match_string: &str) -> bool {
+        string.ends_with(match_string)
+    }
+
     /// Find the specified `character` in the string, starting from the specified `start` position,
     /// and return the first index where it is found.
     /// If the `character` is not found, `-1` is returned.
@@ -1150,19 +1249,25 @@ mod string_functions {
         /// print(text.split(-99));     // prints ["", "hello, world!"]
         /// ```
         #[rhai_fn(name = "split")]
-        pub fn split_at(ctx: NativeCallContext, string: ImmutableString, index: INT) -> Array {
+        pub fn split_at(ctx: NativeCallContext, string: &mut ImmutableString, index: INT) -> Array {
             if index <= 0 {
                 if let Some(n) = index.checked_abs() {
                     let num_chars = string.chars().count();
                     if n as usize > num_chars {
-                        vec![ctx.engine().const_empty_string().into(), string.into()]
+                        vec![
+                            ctx.engine().const_empty_string().into(),
+                            string.as_str().into(),
+                        ]
                     } else {
                         let prefix: String = string.chars().take(num_chars - n as usize).collect();
                         let prefix_len = prefix.len();
                         vec![prefix.into(), string[prefix_len..].into()]
                     }
                 } else {
-                    vec![ctx.engine().const_empty_string().into(), string.into()]
+                    vec![
+                        ctx.engine().const_empty_string().into(),
+                        string.as_str().into(),
+                    ]
                 }
             } else {
                 let prefix: String = string.chars().take(index as usize).collect();
