@@ -10,8 +10,7 @@ use std::fmt;
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 
-/// _(internals)_ Error encountered when tokenizing the script text.
-/// Exported under the `internals` feature only.
+/// Error encountered when tokenizing the script text.
 #[derive(Debug, Eq, PartialEq, Clone, Hash)]
 #[non_exhaustive]
 pub enum LexError {
@@ -52,7 +51,7 @@ impl fmt::Display for LexError {
             Self::ImproperSymbol(s, d) if d.is_empty() => {
                 write!(f, "Invalid symbol encountered: '{}'", s)
             }
-            Self::ImproperSymbol(_, d) => f.write_str(d),
+            Self::ImproperSymbol(.., d) => f.write_str(d),
         }
     }
 }
@@ -89,21 +88,12 @@ pub enum ParseErrorType {
     MalformedCallExpr(String),
     /// An expression in indexing brackets `[]` has syntax error. Wrapped value is the error
     /// description (if any).
-    ///
-    /// Never appears under the `no_index` feature.
     MalformedIndexExpr(String),
-    /// An expression in an `in` expression has syntax error. Wrapped value is the error description
-    /// (if any).
-    ///
-    /// Never appears under the `no_object` and `no_index` features combination.
+    /// An expression in an `in` expression has syntax error. Wrapped value is the error description (if any).
     MalformedInExpr(String),
     /// A capturing  has syntax error. Wrapped value is the error description (if any).
-    ///
-    /// Never appears under the `no_closure` feature.
     MalformedCapture(String),
     /// A map definition has duplicated property names. Wrapped value is the property name.
-    ///
-    /// Never appears under the `no_object` feature.
     DuplicatedProperty(String),
     /// A `switch` case is duplicated.
     DuplicatedSwitchCase,
@@ -116,11 +106,11 @@ pub enum ParseErrorType {
     /// The case condition of a `switch` statement is not appropriate.
     WrongSwitchCaseCondition,
     /// Missing a property name for custom types and maps.
-    ///
-    /// Never appears under the `no_object` feature.
     PropertyExpected,
     /// Missing a variable name after the `let`, `const`, `for` or `catch` keywords.
     VariableExpected,
+    /// Forbidden variable name.  Wrapped value is the variable name.
+    ForbiddenVariable(String),
     /// An identifier is a reserved symbol.
     Reserved(String),
     /// An expression is of the wrong type.
@@ -129,44 +119,32 @@ pub enum ParseErrorType {
     /// Missing an expression. Wrapped value is the expression type.
     ExprExpected(String),
     /// Defining a doc-comment in an appropriate place (e.g. not at global level).
-    ///
-    /// Never appears under the `no_function` feature.
     WrongDocComment,
     /// Defining a function `fn` in an appropriate place (e.g. inside another function).
-    ///
-    /// Never appears under the `no_function` feature.
     WrongFnDefinition,
     /// Defining a function with a name that conflicts with an existing function.
     /// Wrapped values are the function name and number of parameters.
-    ///
-    /// Never appears under the `no_object` feature.
     FnDuplicatedDefinition(String, usize),
     /// Missing a function name after the `fn` keyword.
-    ///
-    /// Never appears under the `no_function` feature.
     FnMissingName,
     /// A function definition is missing the parameters list. Wrapped value is the function name.
-    ///
-    /// Never appears under the `no_function` feature.
     FnMissingParams(String),
     /// A function definition has duplicated parameters. Wrapped values are the function name and
     /// parameter name.
-    ///
-    /// Never appears under the `no_function` feature.
     FnDuplicatedParam(String, String),
     /// A function definition is missing the body. Wrapped value is the function name.
-    ///
-    /// Never appears under the `no_function` feature.
     FnMissingBody(String),
     /// Export statement not at global level.
-    ///
-    /// Never appears under the `no_module` feature.
     WrongExport,
     /// Assignment to an a constant variable. Wrapped value is the constant variable name.
     AssignmentToConstant(String),
     /// Assignment to an inappropriate LHS (left-hand-side) expression.
     /// Wrapped value is the error message (if any).
     AssignmentToInvalidLHS(String),
+    /// A variable is already defined.
+    ///
+    /// Only appears when variables shadowing is disabled.
+    VariableExists(String),
     /// A variable is not found.
     ///
     /// Only appears when strict variables mode is enabled.
@@ -174,16 +152,10 @@ pub enum ParseErrorType {
     /// An imported module is not found.
     ///
     /// Only appears when strict variables mode is enabled.
-    ///
-    /// Never appears under the `no_module` feature.
     ModuleUndefined(String),
     /// Expression exceeding the maximum levels of complexity.
-    ///
-    /// Never appears under the `unchecked` feature.
     ExprTooDeep,
     /// Literal exceeding the maximum size. Wrapped values are the data type name and the maximum size.
-    ///
-    /// Never appears under the `unchecked` feature.
     LiteralTooLarge(String, usize),
     /// Break statement not inside a loop.
     LoopBreak,
@@ -241,6 +213,7 @@ impl fmt::Display for ParseErrorType {
             Self::DuplicatedSwitchCase => f.write_str("Duplicated switch case"),
             Self::DuplicatedVariable(s) => write!(f, "Duplicated variable name: {}", s),
 
+            Self::VariableExists(s) => write!(f, "Variable already defined: {}", s),
             Self::VariableUndefined(s) => write!(f, "Undefined variable: {}", s),
             Self::ModuleUndefined(s) => write!(f, "Undefined module: {}", s),
 
@@ -269,6 +242,7 @@ impl fmt::Display for ParseErrorType {
             Self::WrongSwitchCaseCondition => f.write_str("This switch case cannot have a condition"),
             Self::PropertyExpected => f.write_str("Expecting name of a property"),
             Self::VariableExpected => f.write_str("Expecting name of a variable"),
+            Self::ForbiddenVariable(s) => write!(f, "Forbidden variable name: {}", s),
             Self::WrongFnDefinition => f.write_str("Function definitions must be at global level and cannot be inside a block or another function"),
             Self::FnMissingName => f.write_str("Expecting function name in function declaration"),
             Self::WrongDocComment => f.write_str("Doc-comment must be followed immediately by a function definition"),
@@ -293,7 +267,12 @@ impl From<LexError> for ParseErrorType {
 
 /// Error when parsing a script.
 #[derive(Debug, Eq, PartialEq, Clone, Hash)]
-pub struct ParseError(pub Box<ParseErrorType>, pub Position);
+pub struct ParseError(
+    /// Parse error type.
+    pub Box<ParseErrorType>,
+    /// [Position] of the parse error.
+    pub Position,
+);
 
 impl Error for ParseError {}
 
