@@ -1,7 +1,7 @@
 //! Module that defines the [`Scope`] type representing a function call-stack scope.
 
 use super::dynamic::{AccessMode, Variant};
-use crate::{Dynamic, Identifier, StaticVec};
+use crate::{Dynamic, Identifier};
 use smallvec::SmallVec;
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
@@ -37,10 +37,11 @@ const SCOPE_ENTRIES_INLINED: usize = 8;
 ///
 /// my_scope.push("z", 40_i64);
 ///
-/// engine.eval_with_scope::<()>(&mut my_scope, "let x = z + 1; z = 0;")?;
+/// engine.run_with_scope(&mut my_scope, "let x = z + 1; z = 0;")?;
 ///
-/// assert_eq!(engine.eval_with_scope::<i64>(&mut my_scope, "x + 1")?, 42);
+/// let result: i64 = engine.eval_with_scope(&mut my_scope, "x + 1")?;
 ///
+/// assert_eq!(result, 42);
 /// assert_eq!(my_scope.get_value::<i64>("x").expect("x should exist"), 41);
 /// assert_eq!(my_scope.get_value::<i64>("z").expect("z should exist"), 0);
 /// # Ok(())
@@ -52,20 +53,20 @@ const SCOPE_ENTRIES_INLINED: usize = 8;
 //
 // # Implementation Notes
 //
-// [`Scope`] is implemented as two [`Vec`]'s of exactly the same length.  Variables data (name,
-// type, etc.) is manually split into two equal-length arrays.  That's because variable names take
-// up the most space, with [`Identifier`] being four words long, but in the vast majority of
-// cases the name is NOT used to look up a variable.  Variable lookup is usually via direct
-// indexing, by-passing the name altogether.
+// [`Scope`] is implemented as three arrays of exactly the same length. That's because variable
+// names take up the most space, with [`Identifier`] being three words long, but in the vast
+// majority of cases the name is NOT used to look up a variable.  Variable lookup is usually via
+// direct indexing, by-passing the name altogether.
 //
-// Since [`Dynamic`] is reasonably small, packing it tightly improves cache locality when variables
-// are accessed.
+// [`Dynamic`] is reasonably small so packing it tightly improves cache performance.
 #[derive(Debug, Clone, Hash, Default)]
 pub struct Scope<'a> {
     /// Current value of the entry.
     values: SmallVec<[Dynamic; SCOPE_ENTRIES_INLINED]>,
-    /// (Name, aliases) of the entry.
-    names: SmallVec<[(Identifier, Option<Box<StaticVec<Identifier>>>); SCOPE_ENTRIES_INLINED]>,
+    /// Name of the entry.
+    names: SmallVec<[Identifier; SCOPE_ENTRIES_INLINED]>,
+    /// Aliases of the entry.
+    aliases: SmallVec<[Vec<Identifier>; SCOPE_ENTRIES_INLINED]>,
     /// Phantom to keep the lifetime parameter in order not to break existing code.
     phantom: PhantomData<&'a ()>,
 }
@@ -76,15 +77,12 @@ impl IntoIterator for Scope<'_> {
 
     #[inline]
     fn into_iter(self) -> Self::IntoIter {
-        Box::new(self.values.into_iter().zip(self.names.into_iter()).map(
-            |(value, (name, alias))| {
-                (
-                    name.into(),
-                    value,
-                    alias.map(|a| a.to_vec()).unwrap_or_default(),
-                )
-            },
-        ))
+        Box::new(
+            self.values
+                .into_iter()
+                .zip(self.names.into_iter().zip(self.aliases.into_iter()))
+                .map(|(value, (name, alias))| (name.into(), value, alias)),
+        )
     }
 }
 
@@ -107,6 +105,7 @@ impl Scope<'_> {
         Self {
             values: SmallVec::new_const(),
             names: SmallVec::new_const(),
+            aliases: SmallVec::new_const(),
             phantom: PhantomData,
         }
     }
@@ -133,6 +132,7 @@ impl Scope<'_> {
     pub fn clear(&mut self) -> &mut Self {
         self.names.clear();
         self.values.clear();
+        self.aliases.clear();
         self
     }
     /// Get the number of entries inside the [`Scope`].
@@ -185,7 +185,7 @@ impl Scope<'_> {
     /// ```
     #[inline(always)]
     pub fn push(&mut self, name: impl Into<Identifier>, value: impl Variant + Clone) -> &mut Self {
-        self.push_dynamic_value(name, AccessMode::ReadWrite, Dynamic::from(value))
+        self.push_entry(name, AccessMode::ReadWrite, Dynamic::from(value))
     }
     /// Add (push) a new [`Dynamic`] entry to the [`Scope`].
     ///
@@ -201,7 +201,7 @@ impl Scope<'_> {
     /// ```
     #[inline(always)]
     pub fn push_dynamic(&mut self, name: impl Into<Identifier>, value: Dynamic) -> &mut Self {
-        self.push_dynamic_value(name, value.access_mode(), value)
+        self.push_entry(name, value.access_mode(), value)
     }
     /// Add (push) a new constant to the [`Scope`].
     ///
@@ -224,7 +224,7 @@ impl Scope<'_> {
         name: impl Into<Identifier>,
         value: impl Variant + Clone,
     ) -> &mut Self {
-        self.push_dynamic_value(name, AccessMode::ReadOnly, Dynamic::from(value))
+        self.push_entry(name, AccessMode::ReadOnly, Dynamic::from(value))
     }
     /// Add (push) a new constant with a [`Dynamic`] value to the Scope.
     ///
@@ -247,17 +247,18 @@ impl Scope<'_> {
         name: impl Into<Identifier>,
         value: Dynamic,
     ) -> &mut Self {
-        self.push_dynamic_value(name, AccessMode::ReadOnly, value)
+        self.push_entry(name, AccessMode::ReadOnly, value)
     }
     /// Add (push) a new entry with a [`Dynamic`] value to the [`Scope`].
     #[inline]
-    pub(crate) fn push_dynamic_value(
+    pub(crate) fn push_entry(
         &mut self,
         name: impl Into<Identifier>,
         access: AccessMode,
         mut value: Dynamic,
     ) -> &mut Self {
-        self.names.push((name.into(), None));
+        self.names.push(name.into());
+        self.aliases.push(Vec::new());
         value.set_access_mode(access);
         self.values.push(value);
         self
@@ -292,6 +293,7 @@ impl Scope<'_> {
     pub fn rewind(&mut self, size: usize) -> &mut Self {
         self.names.truncate(size);
         self.values.truncate(size);
+        self.aliases.truncate(size);
         self
     }
     /// Does the [`Scope`] contain the entry?
@@ -310,7 +312,7 @@ impl Scope<'_> {
     #[inline]
     #[must_use]
     pub fn contains(&self, name: &str) -> bool {
-        self.names.iter().any(|(key, _)| name == key)
+        self.names.iter().any(|key| name == key)
     }
     /// Find an entry in the [`Scope`], starting from the last.
     #[inline]
@@ -322,7 +324,7 @@ impl Scope<'_> {
             .iter()
             .rev() // Always search a Scope in reverse order
             .enumerate()
-            .find_map(|(i, (key, _))| {
+            .find_map(|(i, key)| {
                 if name == key {
                     let index = len - 1 - i;
                     Some((index, self.values[index].access_mode()))
@@ -352,8 +354,8 @@ impl Scope<'_> {
             .iter()
             .rev()
             .enumerate()
-            .find(|(_, (key, _))| name == key)
-            .and_then(|(index, _)| self.values[len - 1 - index].flatten_clone().try_cast())
+            .find(|(.., key)| &name == key)
+            .and_then(|(index, ..)| self.values[len - 1 - index].flatten_clone().try_cast())
     }
     /// Check if the named entry in the [`Scope`] is constant.
     ///
@@ -374,9 +376,11 @@ impl Scope<'_> {
     /// ```
     #[inline]
     pub fn is_constant(&self, name: &str) -> Option<bool> {
-        self.get_index(name).and_then(|(_, access)| match access {
-            AccessMode::ReadWrite => None,
-            AccessMode::ReadOnly => Some(true),
+        self.get_index(name).and_then(|(.., access)| {
+            Some(match access {
+                AccessMode::ReadWrite => false,
+                AccessMode::ReadOnly => true,
+            })
         })
     }
     /// Update the value of the named entry in the [`Scope`] if it already exists and is not constant.
@@ -411,7 +415,7 @@ impl Scope<'_> {
         value: impl Variant + Clone,
     ) -> &mut Self {
         match self.get_index(name.as_ref()) {
-            None | Some((_, AccessMode::ReadOnly)) => {
+            None | Some((.., AccessMode::ReadOnly)) => {
                 self.push(name, value);
             }
             Some((index, AccessMode::ReadWrite)) => {
@@ -453,7 +457,7 @@ impl Scope<'_> {
             None => {
                 self.push(name, value);
             }
-            Some((_, AccessMode::ReadOnly)) => panic!("variable {} is constant", name.as_ref()),
+            Some((.., AccessMode::ReadOnly)) => panic!("variable {} is constant", name.as_ref()),
             Some((index, AccessMode::ReadWrite)) => {
                 let value_ref = self.values.get_mut(index).unwrap();
                 *value_ref = Dynamic::from(value);
@@ -510,19 +514,37 @@ impl Scope<'_> {
     /// Panics if the index is out of bounds.
     #[cfg(not(feature = "no_module"))]
     #[inline]
-    pub(crate) fn add_entry_alias(&mut self, index: usize, alias: Identifier) -> &mut Self {
-        let (_, aliases) = self.names.get_mut(index).unwrap();
-        match aliases {
-            None => {
-                let mut list = StaticVec::new_const();
-                list.push(alias);
-                *aliases = Some(list.into());
-            }
-            Some(aliases) if !aliases.iter().any(|a| a == &alias) => aliases.push(alias),
-            Some(_) => (),
+    pub(crate) fn add_alias_by_index(&mut self, index: usize, alias: Identifier) -> &mut Self {
+        let aliases = self.aliases.get_mut(index).unwrap();
+        if aliases.is_empty() || !aliases.contains(&alias) {
+            aliases.push(alias);
         }
         self
     }
+    /// Add an alias to a variable in the [`Scope`] so that it is exported under that name.
+    /// This is an advanced API.
+    ///
+    /// If the alias is empty, then the variable is exported under its original name.
+    ///
+    /// Multiple aliases can be added to any variable.
+    ///
+    /// Only the last variable matching the name (and not other shadowed versions) is aliased by
+    /// this call.
+    #[cfg(not(feature = "no_module"))]
+    #[inline]
+    pub fn set_alias(
+        &mut self,
+        name: impl AsRef<str> + Into<Identifier>,
+        alias: impl Into<Identifier>,
+    ) {
+        if let Some((index, ..)) = self.get_index(name.as_ref()) {
+            let alias = match alias.into() {
+                x if x.is_empty() => name.into(),
+                x => x,
+            };
+            self.add_alias_by_index(index, alias);
+        }
+    }
     /// Clone the [`Scope`], keeping only the last instances of each variable name.
     /// Shadowed variables are omitted in the copy.
     #[inline]
@@ -530,16 +552,23 @@ impl Scope<'_> {
     pub fn clone_visible(&self) -> Self {
         let len = self.len();
 
-        self.names.iter().rev().enumerate().fold(
-            Self::new(),
-            |mut entries, (index, (name, alias))| {
-                if !entries.names.iter().any(|(key, _)| key == name) {
-                    entries.names.push((name.clone(), alias.clone()));
-                    entries.values.push(self.values[len - 1 - index].clone());
+        self.names
+            .iter()
+            .rev()
+            .enumerate()
+            .fold(Self::new(), |mut entries, (index, name)| {
+                if entries.names.is_empty() || !entries.names.contains(name) {
+                    let orig_value = &self.values[len - 1 - index];
+                    let alias = &self.aliases[len - 1 - index];
+                    let mut value = orig_value.clone();
+                    value.set_access_mode(orig_value.access_mode());
+
+                    entries.names.push(name.clone());
+                    entries.values.push(value);
+                    entries.aliases.push(alias.clone());
                 }
                 entries
-            },
-        )
+            })
     }
     /// Get an iterator to entries in the [`Scope`].
     #[inline]
@@ -547,10 +576,8 @@ impl Scope<'_> {
     pub(crate) fn into_iter(self) -> impl Iterator<Item = (Identifier, Dynamic, Vec<Identifier>)> {
         self.names
             .into_iter()
-            .zip(self.values.into_iter())
-            .map(|((name, alias), value)| {
-                (name, value, alias.map(|a| a.to_vec()).unwrap_or_default())
-            })
+            .zip(self.values.into_iter().zip(self.aliases.into_iter()))
+            .map(|(name, (value, alias))| (name, value, alias))
     }
     /// Get an iterator to entries in the [`Scope`].
     /// Shared values are flatten-cloned.
@@ -589,7 +616,17 @@ impl Scope<'_> {
         self.names
             .iter()
             .zip(self.values.iter())
-            .map(|((name, _), value)| (name.as_ref(), value.is_read_only(), value))
+            .map(|(name, value)| (name.as_ref(), value.is_read_only(), value))
+    }
+    /// Get a reverse iterator to entries in the [`Scope`].
+    /// Shared values are not expanded.
+    #[inline]
+    pub(crate) fn iter_rev_raw(&self) -> impl Iterator<Item = (&str, bool, &Dynamic)> {
+        self.names
+            .iter()
+            .rev()
+            .zip(self.values.iter().rev())
+            .map(|(name, value)| (name.as_ref(), value.is_read_only(), value))
     }
     /// Remove a range of entries within the [`Scope`].
     ///
@@ -601,15 +638,16 @@ impl Scope<'_> {
     pub(crate) fn remove_range(&mut self, start: usize, len: usize) {
         self.values.drain(start..start + len).for_each(|_| {});
         self.names.drain(start..start + len).for_each(|_| {});
+        self.aliases.drain(start..start + len).for_each(|_| {});
     }
 }
 
 impl<K: Into<Identifier>> Extend<(K, Dynamic)> for Scope<'_> {
     #[inline]
     fn extend<T: IntoIterator<Item = (K, Dynamic)>>(&mut self, iter: T) {
-        iter.into_iter().for_each(|(name, value)| {
-            self.push_dynamic_value(name, AccessMode::ReadWrite, value);
-        });
+        for (name, value) in iter {
+            self.push_entry(name, AccessMode::ReadWrite, value);
+        }
     }
 }
 
@@ -625,8 +663,8 @@ impl<K: Into<Identifier>> FromIterator<(K, Dynamic)> for Scope<'_> {
 impl<K: Into<Identifier>> Extend<(K, bool, Dynamic)> for Scope<'_> {
     #[inline]
     fn extend<T: IntoIterator<Item = (K, bool, Dynamic)>>(&mut self, iter: T) {
-        iter.into_iter().for_each(|(name, is_constant, value)| {
-            self.push_dynamic_value(
+        for (name, is_constant, value) in iter {
+            self.push_entry(
                 name,
                 if is_constant {
                     AccessMode::ReadOnly
@@ -635,7 +673,7 @@ impl<K: Into<Identifier>> Extend<(K, bool, Dynamic)> for Scope<'_> {
                 },
                 value,
             );
-        });
+        }
     }
 }
 
