@@ -73,8 +73,8 @@ fn test_fn_ptr() -> Result<(), Box<EvalAltResult>> {
             "#
             )
             .expect_err("should error"),
-        EvalAltResult::ErrorInFunctionCall(fn_name, _, err, _)
-            if fn_name == "foo" && matches!(*err, EvalAltResult::ErrorUnboundThis(_))
+        EvalAltResult::ErrorInFunctionCall(fn_name, _, err, ..)
+            if fn_name == "foo" && matches!(*err, EvalAltResult::ErrorUnboundThis(..))
     ));
 
     Ok(())
