@@ -53,7 +53,7 @@ b`: 1}; y["a\nb"]
         *engine
             .eval::<INT>("let y = #{`a${1}`: 1}; y.a1")
             .expect_err("should error"),
-        EvalAltResult::ErrorParsing(ParseErrorType::PropertyExpected, _)
+        EvalAltResult::ErrorParsing(ParseErrorType::PropertyExpected, ..)
     ));
 
     assert!(engine.eval::<bool>(r#"let y = #{a: 1, b: 2, c: 3}; "c" in y"#)?);
@@ -107,6 +107,23 @@ b`: 1}; y["a\nb"]
     Ok(())
 }
 
+#[test]
+fn test_map_prop() -> Result<(), Box<EvalAltResult>> {
+    let mut engine = Engine::new();
+
+    assert_eq!(engine.eval::<()>("let x = #{a: 42}; x.b")?, ());
+
+    engine.set_fail_on_invalid_map_property(true);
+
+    assert!(
+        matches!(*engine.eval::<()>("let x = #{a: 42}; x.b").expect_err("should error"),
+            EvalAltResult::ErrorPropertyNotFound(prop, _) if prop == "b"
+        )
+    );
+
+    Ok(())
+}
+
 #[test]
 fn test_map_assign() -> Result<(), Box<EvalAltResult>> {
     let engine = Engine::new();
@@ -205,7 +222,7 @@ fn test_map_json() -> Result<(), Box<EvalAltResult>> {
 
     assert!(matches!(
         *engine.parse_json("   123", true).expect_err("should error"),
-        EvalAltResult::ErrorParsing(ParseErrorType::MissingToken(token, _), _)
+        EvalAltResult::ErrorParsing(ParseErrorType::MissingToken(token, ..), ..)
             if token == "{"
     ));
 
