@@ -53,9 +53,13 @@ fn test_module_sub_module() -> Result<(), Box<EvalAltResult>> {
 
     assert_eq!(m2.get_var_value::<INT>("answer").unwrap(), 41);
 
+    module.set_custom_type::<()>("Don't Panic");
+
     let mut engine = Engine::new();
     engine.register_static_module("question", module.into());
 
+    assert_eq!(engine.eval::<String>("type_of(())")?, "Don't Panic");
+
     assert_eq!(engine.eval::<INT>("question::MYSTIC_NUMBER")?, 42);
     assert!(engine.eval::<INT>("MYSTIC_NUMBER").is_err());
     assert_eq!(engine.eval::<INT>("question::life::universe::answer")?, 41);
@@ -214,7 +218,7 @@ fn test_module_resolver() -> Result<(), Box<EvalAltResult>> {
                     "#
                 )
                 .expect_err("should error"),
-            EvalAltResult::ErrorTooManyModules(_)
+            EvalAltResult::ErrorTooManyModules(..)
         ));
 
         #[cfg(not(feature = "no_function"))]
@@ -237,7 +241,7 @@ fn test_module_resolver() -> Result<(), Box<EvalAltResult>> {
                     "#
                 )
                 .expect_err("should error"),
-            EvalAltResult::ErrorInFunctionCall(fn_name, _, _, _) if fn_name == "foo"
+            EvalAltResult::ErrorInFunctionCall(fn_name, _, ..) if fn_name == "foo"
         ));
 
         engine.set_max_modules(1000);
@@ -317,11 +321,10 @@ fn test_module_from_ast() -> Result<(), Box<EvalAltResult>> {
             foo = calc(foo);
             hello = `hello, ${foo} worlds!`;
 
-            export
-                x as abc,
-                x as xxx,
-                foo,
-                hello;
+            export x as abc;
+            export x as xxx;
+            export foo;
+            export hello;
         "#,
     )?;
 
@@ -370,7 +373,7 @@ fn test_module_from_ast() -> Result<(), Box<EvalAltResult>> {
         *engine
             .run(r#"import "testing" as ttt; ttt::hidden()"#)
             .expect_err("should error"),
-        EvalAltResult::ErrorFunctionNotFound(fn_name, _) if fn_name == "ttt::hidden ()"
+        EvalAltResult::ErrorFunctionNotFound(fn_name, ..) if fn_name == "ttt::hidden ()"
     ));
 
     Ok(())
@@ -382,13 +385,13 @@ fn test_module_export() -> Result<(), Box<EvalAltResult>> {
 
     assert!(matches!(
         engine.compile("let x = 10; { export x; }").expect_err("should error"),
-        ParseError(x, _) if *x == ParseErrorType::WrongExport
+        ParseError(x, ..) if *x == ParseErrorType::WrongExport
     ));
 
     #[cfg(not(feature = "no_function"))]
     assert!(matches!(
         engine.compile("fn abc(x) { export x; }").expect_err("should error"),
-        ParseError(x, _) if *x == ParseErrorType::WrongExport
+        ParseError(x, ..) if *x == ParseErrorType::WrongExport
     ));
 
     Ok(())
@@ -515,3 +518,47 @@ fn test_module_file() -> Result<(), Box<EvalAltResult>> {
     Module::eval_ast_as_new(Scope::new(), &ast, &engine)?;
     Ok(())
 }
+
+#[cfg(not(feature = "no_function"))]
+#[test]
+fn test_module_environ() -> Result<(), Box<EvalAltResult>> {
+    let mut engine = Engine::new();
+
+    let ast = engine.compile(
+        r#"
+            const SECRET = 42;
+
+            fn foo(x) {
+                print(global::SECRET);
+                global::SECRET + x
+            }
+        "#,
+    )?;
+
+    let mut m = Module::eval_ast_as_new(Scope::new(), &ast, &engine)?;
+
+    m.set_id("test");
+    m.build_index();
+
+    engine.register_static_module("test", m.into());
+
+    assert_eq!(
+        engine.eval::<String>(
+            r#"
+                const SECRET = "hello";
+
+                fn foo(x) {
+                    print(global::SECRET);
+                    global::SECRET + x
+                }
+
+                let t = test::foo(0);
+
+                foo(t)
+            "#
+        )?,
+        "hello42"
+    );
+
+    Ok(())
+}
